package de.osci.helper;

import java.io.FilterInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.PipedInputStream;
import java.io.PipedOutputStream;
import java.security.NoSuchAlgorithmException;
import java.util.Vector;

import javax.xml.parsers.ParserConfigurationException;

import org.xml.sax.SAXException;


/**
 * Diese Klasse fhrt die Kanonisierung gem der Spezifikation
 * http://www.w3.org/TR/xml-c14n durch. Die Funktion beschrnkt
 * sich auf die Anforderungen der OSCI 1.2 Transportbibliothek.
 *
 * <p>Copyright  2010 Freie Hansestadt Bremen</p>
 * <p>Erstellt von bremen online services Entwicklungs- und
 * Betriebsgesellschaft GmbH & Co. KG</p>
 * <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
 * freie Softwarebibliotheken genutzt werden.</p>Die Lizenzbestimmungen knnen
 * unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
 * Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
 * 28015 Bremen schriftlich angefordert werden.</p>
 *
 *
 * @author H. Tabrizi / N. Bngener
 * @version 1.5
 */
public class Canonizer extends FilterInputStream
{
  //  private static Log log = LogFactory.getLog(Canonizer.class);
  // Constants
  /** Namespaces feature id (http://xml.org/sax/features/namespaces). */
  protected static final String NAMESPACES_FEATURE_ID = "http://xml.org/sax/features/namespaces";
  /** Namespace-Prefix feature id (http://xml.org/sax/features/namespaces). */
  protected static final String NAMESPACE_PREFIXES_FEATURE_ID = "http://xml.org/sax/features/namespace-prefixes";
  /** Validation feature id (http://xml.org/sax/features/validation). */
  protected static final String VALIDATION_FEATURE_ID = "http://xml.org/sax/features/validation";
  /** Schema validation feature id (http://apache.org/xml/features/validation/schema). */
  protected static final String SCHEMA_VALIDATION_FEATURE_ID = "http://apache.org/xml/features/validation/schema";
  /** Schema full checking feature id (http://apache.org/xml/features/validation/schema-full-checking). */
  protected static final String SCHEMA_FULL_CHECKING_FEATURE_ID = "http://apache.org/xml/features/validation/schema-full-checking";
  /** Entity External General checking feature id (http://apache.org/xml/features/validation/schema-full-checking). */
  protected static final String EXTERNAL_GENERAL_ENTITIES_FEATURE_ID = "http://xml.org/sax/features/external-general-entities";
  /** Entity External Parameter checking feature id (http://apache.org/xml/features/validation/schema-full-checking). */
  protected static final String EXTERNAL_PARAMETER_ENTITIES_FEATURE_ID = "http://xml.org/sax/features/external-parameter-entities";

  // Data
  private PipedInputStream pis;
  private PipedOutputStream pos;
  private Exception canException;
  private InputStream input;
  private CanParser cp;

  /**
   * Creates a new Canonizer object.
   *
   * @param in undocumented
   * @param sis undocumented
   *
   * @throws IOException undocumented
   * @throws SAXException undocumented
   * @throws ParserConfigurationException undocumented
   * @throws NoSuchAlgorithmException undocumented
   */
  public Canonizer(InputStream in, StoreInputStream sis)
            throws IOException,
                   SAXException,
                   ParserConfigurationException,
                   NoSuchAlgorithmException
  {
    super(in);
    input = in;
    pis = new PipedInputStream();
    pos = new PipedOutputStream(pis);
    cp = new CanParser(pos, sis);
    cp.signedInfos = new Vector();
    cp.signedProperties = new Vector();
    cp.cocoNS = new Vector();
    canException = null;
    new Thread()
      {
        public void run()
        {
          try
          {
            cp.startCanonicalization(input, false);
            pos.close();
          }
          catch (SAXException e)
          {
            canException = e.getException();
          }
          catch (Exception e)
          {
              canException = e;
          }
          finally
          {
            try
            {
              pos.close();
            }
            catch (IOException ex)
            {
              // Keine Aktion, alles zu spt....
            }
          }
        }
      }.start();
  }

  /**
   * undocumented
   *
   * @return undocumented
   *
   * @throws java.io.IOException undocumented
   */
  public int read() throws java.io.IOException
  {
    /*
       byte[] d = new byte[1];
       if (read(d, 0, 1) == -1)
         return -1;
       return ((int)d[0]) & 0xff;
     */
    return pis.read();
  }

  /**
   * undocumented
   *
   * @param b undocumented
   * @param off undocumented
   * @param len undocumented
   *
   * @return undocumented
   *
   * @throws IOException undocumented
   */
  public int read(byte[] b, int off, int len) throws IOException
  {
    return pis.read(b, off, len);
  }

  /**
   * undocumented
   *
   * @return undocumented
   */
  public java.util.Hashtable getDigestValues()
  {
    return cp.digestValues;
  }

  /**
   * undocumented
   *
   * @return undocumented
   */
  public Vector getSignedInfos()
  {
    return cp.signedInfos;
  }

  /**
   * undocumented
   *
   * @return undocumented
   */
  public Vector getSignedProperties()
  {
    return cp.signedProperties;
  }

  /**
   * undocumented
   *
   * @return undocumented
   */
  public Vector getContainerNS()
  {
    return cp.cocoNS;
  }

  /**
   * undocumented
   *
   * @return undocumented
   */
  public Exception getCanException()
  {
    return canException;
  }

}
