package de.osci.helper;

import de.osci.osci12.common.Constants;
import de.osci.osci12.common.DialogHandler;
import de.osci.osci12.encryption.Crypto;

import java.io.FilterInputStream;
import java.io.IOException;
import java.io.InputStream;

import java.security.GeneralSecurityException;

import javax.crypto.Cipher;
import javax.crypto.CipherInputStream;
import javax.crypto.SecretKey;


/**
 * Diese Erweiterung der Klasse java.io.FilterInputStream fhrt eine symmetrische
 * Ver-/Entschlsselung der gelesenen Daten durch. Der Initialisierungsvektor
 * wird gem der XML-Encryption-Spezifikation den Inhaltsdaten hinzugefgt bzw.
 * entnommen.
 *
 * <p>Copyright  2010 Freie Hansestadt Bremen</p>
 * <p>Erstellt von bremen online services Entwicklungs- und
 * Betriebsgesellschaft GmbH & Co. KG</p>
 * <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
 * freie Softwarebibliotheken genutzt werden.</p>Die Lizenzbestimmungen knnen
 * unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
 * Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
 * 28015 Bremen schriftlich angefordert werden.</p>
 *
 *
 * @author R. Lindemann, N.Bngener
 * @version 1.5
 * @see de.osci.helper.SymCipherOutputStream
 */
public class SymCipherInputStream extends FilterInputStream
{
  //  private static Log log = LogFactory.getLog(SymCipherInputStream.class);
  private Cipher cipher;
  private int count = 0;
  byte[] iv;
  private boolean encrypt;
  private byte[] b = new byte[1];

  /**
   * Creates a new SymCipherInputStream object.
   *
   * @param inStream undocumented
   * @param symKey undocumented
   * @param encrypt undocumented
   *
   * @throws IOException undocumented
   * @throws IllegalArgumentException undocumented
   */
  public SymCipherInputStream(InputStream inStream, SecretKey symKey, boolean encrypt)
                       throws IOException
  {
    super(null);
    this.encrypt = encrypt;

    if (inStream == null)
      throw new IllegalArgumentException(DialogHandler.text.getString("invalid_firstargument") + " null");

    if (symKey == null)
      throw new IllegalArgumentException(DialogHandler.text.getString("invalid_secondargument") + " null");

    try
    {
      if (DialogHandler.getSecurityProvider() == null)
        cipher = Cipher.getInstance((String) Constants.JCA_JCE_MAP.get(Crypto.getCipherAlgoID(symKey)));
      else
        cipher = Cipher.getInstance((String) Constants.JCA_JCE_MAP.get(Crypto.getCipherAlgoID(symKey)),
                                    DialogHandler.getSecurityProvider());

      if (encrypt)
      {
        cipher.init(Cipher.ENCRYPT_MODE, symKey);
        iv = cipher.getIV();
      }
      else
      {
        if (symKey.getAlgorithm().startsWith("DESede"))
          iv = new byte[8];
        else
          iv = new byte[16];

        int tmp;

        for (int i = 0; i < iv.length; i++)
        {
          tmp = inStream.read();

          if (tmp == -1)
            throw new IOException("Unexpected EOS");

          iv[i] = (byte) tmp;
        }

        javax.crypto.spec.IvParameterSpec algoParamSpec = new javax.crypto.spec.IvParameterSpec(iv);
        cipher.init(Cipher.DECRYPT_MODE, symKey, algoParamSpec);
      }
    }
    catch (GeneralSecurityException ex)
    {
      if (encrypt)
        throw new IOException(DialogHandler.text.getString("encryption_error"));
      else
        throw new IOException(DialogHandler.text.getString("decryption_error"));
    }

    super.in = new CipherInputStream(inStream, cipher);
  }

  /**
   * undocumented
   *
   * @return undocumented
   *
   * @throws IOException undocumented
   */
  public int read() throws IOException
  {
    if (read(b, 0, 1) == -1)
      return -1;

    return ((int) b[0]) & 0xff;
  }

  /**
   * undocumented
   *
   * @param b undocumented
   * @param off undocumented
   * @param len undocumented
   *
   * @return undocumented
   *
   * @throws IOException undocumented
   */
  public int read(byte[] b, int off, int len) throws IOException
  {
    if (encrypt && (count < iv.length))
    {
      if (len < (iv.length - count))
      {
        System.arraycopy(iv, count, b, off, len);
        count += len;

        return len;
      }
      else
      {
        System.arraycopy(iv, count, b, off, iv.length - count);

        int ret = in.read(b, (off + iv.length) - count, len - iv.length + count);
        ret += (iv.length - count);
        count = iv.length;

        return ret;
      }
    }
    else
    {
      return in.read(b, off, len);
    }
  }
}
