package de.osci.osci12.common;

import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.Hashtable;
import java.util.TimeZone;


/**
 * In dieser Klasse sind Konstanten der Bibliothek definiert.
 * <p>Copyright  2010 Freie Hansestadt Bremen</p>
 * <p>Erstellt von bremen online services Entwicklungs- und
 * Betriebsgesellschaft GmbH & Co. KG</p>
 * <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
 * freie Softwarebibliotheken genutzt werden.</p>Die Lizenzbestimmungen knnen
 * unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
 * Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
 * 28015 Bremen schriftlich angefordert werden.</p>
 *
 *
 * @author R. Lindemann, N.Bngener
 * @version 1.5
 */
public class Constants
{
  /**
   * Hashtable mit den bentigten JCA/JCE-Identifiern
   */
  public final static Hashtable JCA_JCE_MAP = new Hashtable();
  /**
   * Character-Encoding UTF-8
   */
  public final static String CHAR_ENCODING = "UTF-8";
  /**
   * Algorithmus fr Zufallszahlenerstellung wird auf SHA1 Pseudo Random Number Generation gestellt, JCA/JCE-Identifier
   */
  public final static String SECURE_RANDOM_ALGORITHM_SHA1 = "SHA1PRNG";
  /**
   * Algorithmus fr Signaturerstellung, Konstante fr SHA1withRSA
   */
  public final static String SIGNATURE_ALGORITHM_RSA_SHA1 = "http://www.w3.org/2000/09/xmldsig#rsa-sha1";
  /**
   * Algorithmus fr Signaturerstellung, Konstante fr SHA1withRSA, JCA/JCE-Identifier
   */
  private final static String SIGNATURE_ALGORITHM_RSA_SHA1_JCA_JCE = "SHA1withRSA";
  /**
   * Hashtable mit Ablaufdaten der Algorithmen und Schlssellngen
   */
  public static Hashtable OUT_DATES = new Hashtable();
  /**
   * Ablaufdatum fr 1024 Bit RSA-Schlssellnge
   */
  public static Date OUT_DATE_KEYSIZE_1024;
  /**
   * Default-Gltigkeitsdatum fr Algorithmus-Prfungen
   */
  public static Date ACTUAL_DATE;
  /**
   * Algorithmus fr Signaturerstellung, Konstante fr SHA256withRSA
   */
  public final static String SIGNATURE_ALGORITHM_RSA_SHA256 = "http://www.w3.org/2001/04/xmldsig-more#rsa-sha256";
  /**
   * Algorithmus fr Signaturerstellung, Konstante fr SHA256withRSA, JCA/JCE-Identifier
   */
  private final static String SIGNATURE_ALGORITHM_RSA_SHA256_JCA_JCE = "SHA256withRSA";
  /**
   * Algorithmus fr Signaturerstellung, Konstante fr SHA512withRSA
   */
  public final static String SIGNATURE_ALGORITHM_RSA_SHA512 = "http://www.w3.org/2001/04/xmldsig-more#rsa-sha512";
  /**
   * Algorithmus fr Signaturerstellung, Konstante fr SHA512withRSA, JCA/JCE-Identifier
   */
  private final static String SIGNATURE_ALGORITHM_RSA_SHA512_JCA_JCE = "SHA512withRSA";

  /**
   * Algorithmus fr Signaturerstellung, Konstante fr RIPEMD160withRSA
   */
  public final static String SIGNATURE_ALGORITHM_RSA_RIPEMD160 = "http://www.w3.org/2001/04/xmldsig-more/rsa-ripemd160";
  /**
   * Konstante fr Abwrtskompatibilitt nach Korrektur der RIPEMD160withRSA-Konstante (Version 1.5, Typo in RFC 4051)
   */
  public final static String SIGNATURE_ALGORITHM_RSA_RIPEMD160_LEGACY = "http://www.w3.org/2001/04/xmldsig-more#rsa-ripemd160";
  /**
   * Algorithmus fr Signaturerstellung, Konstante fr RIPEMD160withRSA, JCA/JCE-Identifier
   */
  private final static String SIGNATURE_ALGORITHM_RSA_RIPEMD160_JCA_JCE = "RIPEMD160withRSA";
  /**
   * Algorithmus fr Signaturerstellung, Konstante fr SHA256withECDSA
   */
  public static final String SIGNATURE_ALGORITHM_ECDSA_SHA256 = "http://www.w3.org/2001/04/xmldsig-more#ecdsa-sha256";
  /**
   * Algorithmus fr Signaturerstellung, Konstante fr SHA256withECDSA, JCA/JCE-Identifier
   */
  private static final String SIGNATURE_ALGORITHM_ECDSA_SHA256_JCA_JCE = "SHA256withCVC-ECDSA";
  /**
   * Algorithmus fr Signaturerstellung, Konstante fr SHA512withECDSA
   */
  public static final String SIGNATURE_ALGORITHM_ECDSA_SHA512 = "http://www.w3.org/2001/04/xmldsig-more#ecdsa-sha512";
  /**
   * Algorithmus fr Signaturerstellung, Konstante fr SHA512withECDSA, JCA/JCE-Identifier
   */
  private static final String SIGNATURE_ALGORITHM_ECDSA_SHA512_JCA_JCE = "SHA512withCVC-ECDSA";
  /**
   * Algorithmus fr Hashwerterstellung, Konstante fr SHA1
   */
  public final static String DIGEST_ALGORITHM_SHA1 = "http://www.w3.org/2000/09/xmldsig#sha1";
  /**
   * Algorithmus fr Hashwerterstellung, Konstante fr SHA1, JCA/JCE-Identifier
   */
  private final static String DIGEST_ALGORITHM_SHA1_JCA_JCE = "SHA-1";
  /**
   * Algorithmus fr Hashwerterstellung, Konstante fr SHA256
   */
  public final static String DIGEST_ALGORITHM_SHA256 = "http://www.w3.org/2001/04/xmlenc#sha256";
  /**
   * Algorithmus fr Hashwerterstellung, Konstante fr SHA256, JCA/JCE-Identifier
   */
  private final static String DIGEST_ALGORITHM_SHA256_JCA_JCE = "SHA-256";
  /**
   * Algorithmus fr Hashwerterstellung, Konstante fr SHA512
   */
  public final static String DIGEST_ALGORITHM_SHA512 = "http://www.w3.org/2001/04/xmlenc#sha512";
  /**
   * Algorithmus fr Hashwerterstellung, Konstante fr SHA512, JCA/JCE-Identifier
   */
  private final static String DIGEST_ALGORITHM_SHA512_JCA_JCE = "SHA-512";
  /**
   * Algorithmus fr Hashwerterstellung, Konstante fr RIPEMD160
   */
  public final static String DIGEST_ALGORITHM_RIPEMD160 = "http://www.w3.org/2001/04/xmlenc#ripemd160";
  /**
   * Algorithmus fr Hashwerterstellung, Konstante fr SHA1, JCA/JCE-Identifier
   */
  private final static String DIGEST_ALGORITHM_RIPEMD160_JCA_JCE = "RIPEMD160";
  /**
   * Algorithmus fr symmetrische Verschlsselung Two-Key-Triple-DES (CBC-Mode)
   */
  public final static String SYMMETRIC_CIPHER_ALGORITHM_TDES_CBC = "http://www.w3.org/2001/04/xmlenc#tripledes-cbc";
  /**
   * Algorithmus fr symmetrische Verschlsselung Two-Key-Triple-DES (CBC-Mode), JCA/JCE-Identifier
   */
  private final static String SYMMETRIC_CIPHER_ALGORITHM_TDES_CBC_JCA_JCE = "DESede/CBC/PKCS5Padding";
  /**
   * Algorithmus fr symmetrische Verschlsselung AES-128
   */
  public final static String SYMMETRIC_CIPHER_ALGORITHM_AES128 = "http://www.w3.org/2001/04/xmlenc#aes128-cbc";
  /**
   * Algorithmus fr symmetrische Verschlsselung AES (CBC-Mode), JCA/JCE-Identifier
   */
  private final static String SYMMETRIC_CIPHER_ALGORITHM_AES_CBC_JCA_JCE = "AES/CBC/PKCS5Padding";
  /**
   * Algorithmus fr symmetrische Verschlsselung AES-192
   */
  public final static String SYMMETRIC_CIPHER_ALGORITHM_AES192 = "http://www.w3.org/2001/04/xmlenc#aes192-cbc";
  /**
   * Algorithmus fr symmetrische Verschlsselung AES-256
   */
  public final static String SYMMETRIC_CIPHER_ALGORITHM_AES256 = "http://www.w3.org/2001/04/xmlenc#aes256-cbc";
  /**
   * Algorithmus fr asymmetrische Verschlsselung RSAES-PKCS1-v1_5
   */
  public final static String ASYMMETRIC_CIPHER_ALGORITHM_RSA_1_5 = "http://www.w3.org/2001/04/xmlenc#rsa-1_5";
  /**
   * Algorithmus fr asymmetrische Verschlsselung RSAES-PKCS1-v1_5, JCA/JCE-Identifier
   */
  private final static String ASYMMETRIC_CIPHER_ALGORITHM_RSA_1_5_JCA_JCE = "RSA/ECB/PKCS1Padding";
  /**
   * Event-Id 'Verbindung zum Intermedir wird aufgebaut'
   */
  public final static int EVENT_CONNECT = 0;
  /**
   * Event-Id 'Auftragsnachricht wird signiert'
   */
  public final static int EVENT_SIGN_MSG = 1;
  /**
   * Event-Id 'Auftragsnachricht wird versendet'
   */
  public final static int EVENT_SEND_MSG = 2;
  /**
   * Event-Id 'Antwortnachricht wird empfangen'
   */
  public final static int EVENT_RECEIVE_MSG = 3;
  /**
   * Event-Id 'Vorgang abgeschlossen'
   */
  public final static int EVENT_ACTION_COMPLETE = 4;

  // Transforms
  /**
   * XSLT-Transformation
   */
  public final static String TRANSFORM_XSLT = "http://www.w3.org/TR/1999/REC-xslt-19991116";
  /**
   * Kanonisierung
   */
  public final static String TRANSFORM_CANONICALIZATION = "http://www.w3.org/TR/2001/REC-xml-c14n-20010315";
  /**
   * Tranformer fr Base64-Decoder
   */
  public final static String TRANSFORM_BASE64 = "http://www.w3.org/2000/09/xmldsig#base64";

  static
  {
    JCA_JCE_MAP.put(ASYMMETRIC_CIPHER_ALGORITHM_RSA_1_5, ASYMMETRIC_CIPHER_ALGORITHM_RSA_1_5_JCA_JCE);
    JCA_JCE_MAP.put(DIGEST_ALGORITHM_SHA1, DIGEST_ALGORITHM_SHA1_JCA_JCE);
    JCA_JCE_MAP.put(DIGEST_ALGORITHM_SHA256, DIGEST_ALGORITHM_SHA256_JCA_JCE);
    JCA_JCE_MAP.put(DIGEST_ALGORITHM_SHA512, DIGEST_ALGORITHM_SHA512_JCA_JCE);
    JCA_JCE_MAP.put(DIGEST_ALGORITHM_RIPEMD160, DIGEST_ALGORITHM_RIPEMD160_JCA_JCE);
    JCA_JCE_MAP.put(SIGNATURE_ALGORITHM_RSA_SHA1, SIGNATURE_ALGORITHM_RSA_SHA1_JCA_JCE);
    JCA_JCE_MAP.put(SIGNATURE_ALGORITHM_RSA_SHA256, SIGNATURE_ALGORITHM_RSA_SHA256_JCA_JCE);
    JCA_JCE_MAP.put(SIGNATURE_ALGORITHM_RSA_SHA512, SIGNATURE_ALGORITHM_RSA_SHA512_JCA_JCE);
    JCA_JCE_MAP.put(SIGNATURE_ALGORITHM_RSA_RIPEMD160, SIGNATURE_ALGORITHM_RSA_RIPEMD160_JCA_JCE);
    JCA_JCE_MAP.put(SIGNATURE_ALGORITHM_RSA_RIPEMD160_LEGACY, SIGNATURE_ALGORITHM_RSA_RIPEMD160_JCA_JCE);
    JCA_JCE_MAP.put(SIGNATURE_ALGORITHM_ECDSA_SHA256, SIGNATURE_ALGORITHM_ECDSA_SHA256_JCA_JCE);
    JCA_JCE_MAP.put(SIGNATURE_ALGORITHM_ECDSA_SHA512, SIGNATURE_ALGORITHM_ECDSA_SHA512_JCA_JCE);
    JCA_JCE_MAP.put(SYMMETRIC_CIPHER_ALGORITHM_TDES_CBC, SYMMETRIC_CIPHER_ALGORITHM_TDES_CBC_JCA_JCE);
    JCA_JCE_MAP.put(SYMMETRIC_CIPHER_ALGORITHM_AES128, SYMMETRIC_CIPHER_ALGORITHM_AES_CBC_JCA_JCE);
    JCA_JCE_MAP.put(SYMMETRIC_CIPHER_ALGORITHM_AES192, SYMMETRIC_CIPHER_ALGORITHM_AES_CBC_JCA_JCE);
    JCA_JCE_MAP.put(SYMMETRIC_CIPHER_ALGORITHM_AES256, SYMMETRIC_CIPHER_ALGORITHM_AES_CBC_JCA_JCE);

    Calendar c = new GregorianCalendar(TimeZone.getTimeZone("Europe/Berlin"));
    c.clear();
    c.set(2011, 6, 1);
    ACTUAL_DATE = c.getTime();

    c.clear();
    c.set(2008, 6, 1);
    OUT_DATE_KEYSIZE_1024 = c.getTime();

    c.clear();
    c.set(2008, 6, 1); // BSI-Empfehlung
    OUT_DATES.put(DIGEST_ALGORITHM_SHA1, c.getTime());
    OUT_DATES.put(SIGNATURE_ALGORITHM_RSA_SHA1, c.getTime());

    c.clear();
    c.set(2011, 0, 1);
    OUT_DATES.put(DIGEST_ALGORITHM_RIPEMD160, c.getTime());
    OUT_DATES.put(SIGNATURE_ALGORITHM_RSA_RIPEMD160, c.getTime());
    OUT_DATES.put(SIGNATURE_ALGORITHM_RSA_RIPEMD160_LEGACY, c.getTime());

  }
}
