package de.osci.osci12.common;

import de.osci.osci12.OSCIException;
import de.osci.osci12.messagetypes.OSCIMessage;


/**
 * Diese Exception wird fr die in der OSCI 1.2 Transport-Spezifikation
 * definierten Fehlermeldungen auf Auftragsebene verwendet.
 * <p>Copyright  2010 Freie Hansestadt Bremen</p>
 * <p>Erstellt von bremen online services Entwicklungs- und
 * Betriebsgesellschaft GmbH & Co. KG</p>
 * <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
 * freie Softwarebibliotheken genutzt werden.</p>Die Lizenzbestimmungen knnen
 * unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
 * Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
 * 28015 Bremen schriftlich angefordert werden.</p>
 *
 *
 * @author R. Lindemann, N.Bngener
 * @version 1.5
 */
public class OSCIErrorException extends OSCIException
{
  private OSCIMessage message;
	
  /**
   * Legt ein neues OSCIErrorException Objekt an.
   *
   * @param errorCode Fehlercode
   */
  public OSCIErrorException(String errorCode)
  {
    super(errorCode);
  }
  
  /**
   * Legt ein neues OSCIErrorException Objekt an.
   *
   * @param errorCode Fehlercode
   * @param message OSCI-Nachrichtenobjekt, dessen Verarbeitung den Fehler
   * ausgelst hat.
   */
  public OSCIErrorException(String errorCode, OSCIMessage message)
  {
    super(errorCode);
    this.message = message;
  }
  
  /**
   * Liefert das OSCI-Nachrichtenobjekt, dessen Verarbeitung den Fehler
   * ausgelst hat. Wenn beim Auftreten des Fehlers noch kein Nachrichtenobjekt
   * angelegt werden konnte, wir null zurckgegeben. Das Objekt ist in dem
   * Zustand zum Zeitpunkt des Fehlers, also ggf. unvollstndig.
   * @return OSCI-Nachrichtenobjekt, ggf. null
   */
  public OSCIMessage getOSCIMessage()
  {
	  return message;
  }
  
}
