package de.osci.osci12.encryption;

import de.osci.osci12.common.Constants;
import de.osci.osci12.common.DialogHandler;

import java.security.Key;
import java.security.NoSuchAlgorithmException;
import java.security.cert.X509Certificate;
import java.security.interfaces.ECPublicKey;
import java.security.interfaces.RSAPublicKey;

import javax.crypto.Cipher;
import javax.crypto.KeyGenerator;
import javax.crypto.SecretKey;


/**
 * <p>Copyright  2010 Freie Hansestadt Bremen</p>
 * <p>Erstellt von bremen online services Entwicklungs- und
 * Betriebsgesellschaft GmbH & Co. KG</p>
 * <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
 * freie Softwarebibliotheken genutzt werden.</p>Die Lizenzbestimmungen knnen
 * unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
 * Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
 * 28015 Bremen schriftlich angefordert werden.</p>
 *
 *
 * @author R. Lindemann, N.Bngener
 * @version 1.5
 */
public class Crypto
{
  //  private static Log log = LogFactory.getLog(Crypto.class);
  private static String hex = "0123456789abcdef";

  /**
   * undocumented
   *
   * @param algorithm undocumented
   *
   * @return undocumented
   *
   * @throws NoSuchAlgorithmException undocumented
   */
  public static javax.crypto.SecretKey createSymKey(String algorithm)
                                             throws NoSuchAlgorithmException
  {
    String algo = (String) Constants.JCA_JCE_MAP.get(algorithm);
    KeyGenerator keyGenerator;

    if (DialogHandler.getSecurityProvider() == null)
      keyGenerator = javax.crypto.KeyGenerator.getInstance(algo.substring(0, algo.indexOf('/')));
    else
      keyGenerator = javax.crypto.KeyGenerator.getInstance(algo.substring(0, algo.indexOf('/')),
                                                           DialogHandler.getSecurityProvider());

    if (algorithm.equals(Constants.SYMMETRIC_CIPHER_ALGORITHM_AES128))
      keyGenerator.init(128);
    else if (algorithm.equals(Constants.SYMMETRIC_CIPHER_ALGORITHM_AES192))
      keyGenerator.init(192);
    else if (algorithm.equals(Constants.SYMMETRIC_CIPHER_ALGORITHM_AES256))
      keyGenerator.init(256);

    return keyGenerator.generateKey();
  }

  /**
   * undocumented
   *
   * @param data undocumented
   * @param algorithm undocumented
   *
   * @return undocumented
   */
  public static javax.crypto.SecretKey createSymKey(byte[] data, String algorithm)
  {
    String algo = (String) Constants.JCA_JCE_MAP.get(algorithm);

    return new javax.crypto.spec.SecretKeySpec(data, algo.substring(0, algo.indexOf('/')));
  }

  /**
   * undocumented
   *
   * @param b undocumented
   *
   * @return undocumented
   */
  public static String toHex(byte[] b)
  {
    StringBuffer s = new StringBuffer();
    int j;

    for (int i = 0; i < b.length; i++)
    {
      // oberes nibble
      j = (b[i] >> 4) & 0xf;
      s.append(hex.charAt(j));
      // unteres nibble
      j = b[i] & 0xf;
      s.append(hex.charAt(j));
    }

    return s.toString();
  }

  /**
   * undocumented
   *
   * @param key undocumented
   * @param data undocumented
   *
   * @return undocumented
   *
   * @throws OSCICipherException undocumented
   * @throws NoSuchAlgorithmException undocumented
   */
  public static byte[] doRSADecryption(Key key, byte[] data)
                                throws OSCICipherException,
                                       NoSuchAlgorithmException
  {
    try
    {
      javax.crypto.Cipher cipher;

      if (DialogHandler.getSecurityProvider() == null)
        cipher = javax.crypto.Cipher.getInstance((String) Constants.JCA_JCE_MAP.get(Constants.ASYMMETRIC_CIPHER_ALGORITHM_RSA_1_5));
      else
        cipher = javax.crypto.Cipher.getInstance((String) Constants.JCA_JCE_MAP.get(Constants.ASYMMETRIC_CIPHER_ALGORITHM_RSA_1_5),
                                                 DialogHandler.getSecurityProvider());

      cipher.init(Cipher.DECRYPT_MODE, key);

      byte[] cipheredData = cipher.doFinal(data);

      return cipheredData;
    }
    catch (NoSuchAlgorithmException ex)
    {
      throw ex;
    }
    catch (Exception ex)
    {
      throw new OSCICipherException("decryption_error");
    }
  }

  /**
   * undocumented
   *
   * @param encryptionCert undocumented
   * @param key undocumented
   *
   * @return undocumented
   *
   * @throws OSCICipherException undocumented
   * @throws NoSuchAlgorithmException undocumented
   */
  public static byte[] doRSAEncryption(java.security.cert.X509Certificate encryptionCert, Key key)
                                throws OSCICipherException,
                                       NoSuchAlgorithmException
  {
    try
    {
      javax.crypto.Cipher cipher;

      if (DialogHandler.getSecurityProvider() == null)
        cipher = javax.crypto.Cipher.getInstance((String) Constants.JCA_JCE_MAP.get(Constants.ASYMMETRIC_CIPHER_ALGORITHM_RSA_1_5));
      else
        cipher = javax.crypto.Cipher.getInstance((String) Constants.JCA_JCE_MAP.get(Constants.ASYMMETRIC_CIPHER_ALGORITHM_RSA_1_5),
                                                 DialogHandler.getSecurityProvider());

      cipher.init(Cipher.WRAP_MODE, encryptionCert);

      byte[] cipheredData = cipher.wrap(key);

      return cipheredData;
    }
    catch (NoSuchAlgorithmException ex)
    {
      throw ex;
    }
    catch (Exception ex)
    {
      throw new OSCICipherException("encryption_error");
    }
  }

  /**
   * undocumented
   *
   * @param sk undocumented
   *
   * @return undocumented
   */
  public static String getCipherAlgoID(SecretKey sk)
  {
    String algo = null;

    if (sk.getAlgorithm().equals("DESede"))
      algo = Constants.SYMMETRIC_CIPHER_ALGORITHM_TDES_CBC;
    else if (sk.getAlgorithm().equals("AES"))
    {
      if ((sk.getEncoded().length * 8) == 128)
        algo = Constants.SYMMETRIC_CIPHER_ALGORITHM_AES128;
      else if ((sk.getEncoded().length * 8) == 192)
        algo = Constants.SYMMETRIC_CIPHER_ALGORITHM_AES192;
      else if ((sk.getEncoded().length * 8) == 256)
        algo = Constants.SYMMETRIC_CIPHER_ALGORITHM_AES256;
    }

    return algo;
  }

  /**
   * undocumented
   *
   * @param len undocumented
   * @param algorithm undocumented
   *
   * @return undocumented
   */
  public static long calcSymEncLength(long len, String algorithm)
  {
    if (algorithm.equals(Constants.SYMMETRIC_CIPHER_ALGORITHM_TDES_CBC))
      return 16 + ((len / 8) * 8);
    else
      return 32 + ((len / 16) * 16);
  }

  public static int getKeySize(X509Certificate cert)
  {
    if (cert.getPublicKey().getAlgorithm().equals("EC"))
      return ((ECPublicKey)cert.getPublicKey()).getParams().getOrder().bitLength();
    else
      return ((RSAPublicKey)cert.getPublicKey()).getModulus().bitLength();
  }
}
