package de.osci.osci12.encryption;

import de.osci.osci12.common.DialogHandler;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;

import java.io.IOException;
import java.io.OutputStreamWriter;

import java.util.Enumeration;
import java.util.Vector;


/**
 * <p>Copyright  2010 Freie Hansestadt Bremen</p>
 * <p>Erstellt von bremen online services Entwicklungs- und
 * Betriebsgesellschaft GmbH & Co. KG</p>
 * <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
 * freie Softwarebibliotheken genutzt werden.</p>Die Lizenzbestimmungen knnen
 * unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
 * Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
 * 28015 Bremen schriftlich angefordert werden.</p>
 *
 *
 * @author R. Lindemann, N.Bngener
 * @version 1.5
 */
public class EncryptedDataBuilder extends DefaultHandler
{
  private static Log log = LogFactory.getLog(EncryptedDataBuilder.class);
  DefaultHandler parentHandler;
  private Vector transformer = new Vector();
  XMLReader xmlReader;
  private EncryptedData encrytedDataObject;
  private String edId = "";
  private String cipherRefId = "";
  private OutputStreamWriter valueWriter;
  private CipherValue cv;
  protected final static String DS_XMLNS = "http://www.w3.org/2000/09/xmldsig#";
  protected final static String XENC_XMLNS = "http://www.w3.org/2001/04/xmlenc#";

  /**
   * Creates a new EncryptedDataBuilder object.
   *
   * @param xmlReader undocumented
   * @param parentHandler undocumented
   * @param attributes undocumented
   */
  public EncryptedDataBuilder(XMLReader xmlReader, DefaultHandler parentHandler, Attributes attributes)
  {
    this.xmlReader = xmlReader;
    this.parentHandler = parentHandler;
    encrytedDataObject = new EncryptedData("");

    if ((attributes != null) && (attributes.getValue("Id") != null))
      encrytedDataObject.setId(attributes.getValue("Id"));
  }

  /**
   * undocumented
   *
   * @param uri undocumented
   * @param localName undocumented
   * @param qName undocumented
   * @param attributes undocumented
   *
   * @throws SAXException undocumented
   */
  public void startElement(String uri, String localName, String qName, Attributes attributes)
                    throws SAXException
  {
    try
    {
      if (log.isDebugEnabled())
        log.debug("Start-Element: " + localName);

      if (localName.equals("EncryptedData") && uri.equals(XENC_XMLNS))
      {
        if (log.isDebugEnabled())
          log.debug("encryptedData");

        if (encrytedDataObject == null)
          encrytedDataObject = new EncryptedData(edId);

        if (attributes.getValue("Id") != null)
          encrytedDataObject.setId(attributes.getValue("Id"));
      }
      else if (localName.equals("EncryptionMethod") && uri.equals(XENC_XMLNS))
      {
        encrytedDataObject.setEncryptionMethodAlgorithm(attributes.getValue("Algorithm"));
      }
      else if (localName.equals("KeyInfo") && uri.equals(DS_XMLNS))
      {
        DefaultHandler childBuilder = new de.osci.osci12.signature.KeyInfoBuilder(xmlReader, this, attributes);
        xmlReader.setContentHandler(childBuilder);
      }
      else if (localName.equals("CipherData") && uri.equals(XENC_XMLNS))
        ; //nothing to do
      else if (localName.equals("CipherReference") && uri.equals(XENC_XMLNS))
      {
        cipherRefId = attributes.getValue("URI");
      }
      else if (localName.equals("Transforms") && uri.equals(XENC_XMLNS))
        ; // nothing to do
      else if (localName.equals("Transform") && uri.equals(DS_XMLNS))
        transformer.add(attributes.getValue("Algorithm"));
      else if (localName.equals("CipherValue") && uri.equals(XENC_XMLNS))
      {
        cv = new CipherValue();
        valueWriter = cv.getStreamToWrite();
      }
      else
        throw new SAXException(DialogHandler.text.getString("unexpected_entry") + ": " + localName);
    }
    catch (SAXException ex)
    {
      throw ex;
    }
    catch (Exception ex1)
    {
      log.error("Allgemeiner Fehler im Start-Element: " + localName);
      throw new SAXException(ex1);
    }
  }

  /**
   * undocumented
   *
   * @param ch undocumented
   * @param start undocumented
   * @param length undocumented
   *
   * @throws SAXException undocumented
   */
  public void characters(char[] ch, int start, int length)
                  throws SAXException
  {
    //    if (log.isDebugEnabled()) log.debug("Character: " + new String(ch, start, length));
    if (valueWriter == null)
    {
      for (int i = 0; i < length; i++)
      {
        if (ch[start + i] > ' ')
          throw new SAXException(DialogHandler.text.getString("unexpected_char"));
      }
    }
    else
    {
      try
      {
        valueWriter.write(ch, start, length);
      }
      catch (IOException ex)
      {
        throw new SAXException(ex);
      }
    }
  }

  /*  public static void main(String[] args)
     {
       try
       {
         XMLReader reader = null;
         javax.xml.parsers.SAXParserFactory saxFactory = javax.xml.parsers.SAXParserFactory.newInstance();
         SAXParser parser = saxFactory.newSAXParser();
         reader = parser.getXMLReader();
         EncryptedDataBuilder builder = new EncryptedDataBuilder(reader, null, null);
         reader.setFeature("http://xml.org/sax/features/validation", false);
         reader.setContentHandler(builder);
         reader.setErrorHandler(builder);
         reader.parse(new InputSource(builder.getClass().getResourceAsStream("/de/osci/osci12/samples/msg/encrypedValue.xml")));
         ByteArrayOutputStream outStream = new ByteArrayOutputStream();
         builder.getEncryptedData().writeXML(outStream);
         if (log.isDebugEnabled()) log.debug("Fertig ist: " + outStream.toString());
       }
       catch (Exception ex1)
       {
         log.error("", ex1);
       }
     }
   */
  public void endElement(String uri, String localName, String qName)
                  throws SAXException
  {
    if (log.isDebugEnabled())
      log.debug("Ende-Element: " + localName);

    if (localName.equals("EncryptedData") && uri.equals(XENC_XMLNS))
    {
      xmlReader.setContentHandler(parentHandler);
      parentHandler.endElement(uri, localName, qName);
    }
    else if (localName.equals("EncryptionMethod") && uri.equals(XENC_XMLNS))
      ; //nothing  to do
    else if (localName.equals("CipherData") && uri.equals(XENC_XMLNS))
      ; //nothing to do
    else if (localName.equals("EncryptedKey") && uri.equals(XENC_XMLNS))
      ;
    // nothing to do
    else if (localName.equals("CipherData") && uri.equals(XENC_XMLNS))
      ; //nothing to do
    else if (localName.equals("CipherReference") && uri.equals(XENC_XMLNS))
    {
      try
      {
        CipherReference cr = new CipherReference(cipherRefId);
        Enumeration transformers = transformer.elements();

        while (transformers.hasMoreElements())
          cr.addTransform((String) transformers.nextElement());

        encrytedDataObject.setCipherData(new CipherData(cr));
      }
      catch (Exception ex1)
      {
        log.error("Fehler beim setzen der CipherReference.", ex1);
        throw new SAXException(ex1);
      }
    }
    else if (localName.equals("Transforms") && uri.equals(XENC_XMLNS))
      ; // nothing to do
    else if (localName.equals("Transform") && uri.equals(DS_XMLNS))
      ; //nothing to do
    else if (localName.equals("CipherValue") && uri.equals(XENC_XMLNS))
    {
      try
      {
        valueWriter.close();
      }
      catch (IOException ex)
      {
        throw new SAXException(ex);
      }

      encrytedDataObject.setCipherData(new CipherData(cv));
    }
    else
      throw new SAXException(DialogHandler.text.getString("unexpected_entry") + ": " + localName);

    valueWriter = null;
  }

  /**
   * undocumented
   *
   * @return undocumented
   */
  public EncryptedData getEncryptedData()
  {
    return encrytedDataObject;
  }
}
