package de.osci.osci12.encryption;

import de.osci.osci12.OSCIException;
import de.osci.osci12.common.Constants;

import java.io.IOException;
import java.io.OutputStream;


/**
 * <p>Copyright  2010 Freie Hansestadt Bremen</p>
 * <p>Erstellt von bremen online services Entwicklungs- und
 * Betriebsgesellschaft GmbH & Co. KG</p>
 * <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
 * freie Softwarebibliotheken genutzt werden.</p>Die Lizenzbestimmungen knnen
 * unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
 * Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
 * 28015 Bremen schriftlich angefordert werden.</p>
 *
 *
 * @author R. Lindemann, N.Bngener
 * @version 1.5
 */
public class EncryptedKey extends EncryptedType
{
  //  private static Log log = LogFactory.getLog(EncryptedKey.class);
  /** Empfnger der Nachricht */
  private String recipient;
  /** Name dieses Schlssels. Kann durch KeyName referenziert werden. */
  private String carriedKeyName;
  /**
   * Referenz auf alle mit diesem Schluessel verschluesselten EncryptedType
   * Elemente.
   */
  private String referenceList;

  /**
   * Fr den Parser
   * @param encryptionMethodAlgorithm
   */
  EncryptedKey(String encryptionMethodAlgorithm)
  {
    this.setEncryptionMethodAlgorithm(encryptionMethodAlgorithm);
  }

  /**
   * Creates a new EncryptedKey object.
   *
   * @param encryptionMethodAlgorithm undocumented
   * @param cipherValue undocumented
   */
  public EncryptedKey(String encryptionMethodAlgorithm, CipherValue cipherValue)
  {
    this.setEncryptionMethodAlgorithm(encryptionMethodAlgorithm);

    CipherData cipherData = new CipherData(cipherValue);
    this.setCipherData(cipherData);
  }

  /**
   * undocumented
   *
   * @param recipient undocumented
   */
  public void setRecipient(String recipient)
  {
    this.recipient = recipient;
  }

  /**
   * undocumented
   *
   * @return undocumented
   */
  public String getRecipient()
  {
    return recipient;
  }

  /**
   * undocumented
   *
   * @param carriedKeyName undocumented
   */
  public void setCarriedKeyName(String carriedKeyName)
  {
    this.carriedKeyName = carriedKeyName;
  }

  /**
   * undocumented
   *
   * @return undocumented
   */
  public String getCarriedKeyName()
  {
    return carriedKeyName;
  }

  /**
   * undocumented
   *
   * @param referenceList undocumented
   */
  public void setReferenceList(String referenceList)
  {
    this.referenceList = referenceList;
  }

  /**
   * undocumented
   *
   * @return undocumented
   */
  public String getReferenceList()
  {
    return referenceList;
  }

  /**
   * undocumented
   *
   * @param uRI undocumented
   */
  public void addDataReference(String uRI)
  {
  }

  /**
   * undocumented
   *
   * @param uRI undocumented
   */
  public void addKeyReference(String uRI)
  {
  }

  /**
   * undocumented
   *
   * @param out undocumented
   *
   * @throws IOException undocumented
   * @throws OSCIException undocumented
   */
  public void writeXML(OutputStream out, String ds, String xenc)
                throws IOException, 
                       OSCIException
  {
    out.write(("<" + xenc + ":EncryptedKey>").getBytes(Constants.CHAR_ENCODING));
    out.write(("<" + xenc + ":EncryptionMethod Algorithm=\"" + this.getEncryptionMethodAlgorithm() + "\"></" + xenc +
              ":EncryptionMethod>").getBytes(Constants.CHAR_ENCODING));
    this.getKeyInfo().writeXML(out, ds, xenc);
    this.getCipherData().writeXML(out, ds, xenc);
    out.write(("</" + xenc + ":EncryptedKey>").getBytes(Constants.CHAR_ENCODING));
  }
}
