package de.osci.osci12.extinterfaces;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;


/**
 * Implementierungen dieser Klasse knnen mit Hilfe der Methode
 * setDataBuffer(OSCIDataSource buffer) des DialogHandlers installiert
 * werden, falls Inhaltsdaten nicht durch die default-Implementierung
 * SwapBuffer im Arbeitsspeicher bzw. in temporren Dateien gepuffert werden
 * sollen, sondern beispielsweise in einer Datenbank.<br>
 * Dieser Puffer-Mechanismus wird von den Klassen EncryptedData, Content und Attachment
 * genutzt.
 *
 * <p>Copyright  2010 Freie Hansestadt Bremen</p>
 * <p>Erstellt von bremen online services Entwicklungs- und
 * Betriebsgesellschaft GmbH & Co. KG</p>
 * <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
 * freie Softwarebibliotheken genutzt werden.</p>Die Lizenzbestimmungen knnen
 * unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
 * Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
 * 28015 Bremen schriftlich angefordert werden.</p>
 *
 *
 * @author R. Lindemann, N.Bngener
 * @version 1.5
 * @see de.osci.osci12.common.SwapBuffer
 */
public abstract class OSCIDataSource
{
  /**
   * Die Implementierung dieser statischen Methode muss eine neue Instanz der Klasse
   * zurckgeben.
   * @return neue Instanz der implementierenden Klasse
   * @throws IOException wenn ein Fehler auftritt
   */
  public abstract OSCIDataSource newInstance() throws IOException;

  /**
   * Die Implementierung dieser Methode muss einen OutputStream liefern, in den
   * die zu puffernden Daten geschrieben werden knnen.
   * @return den OutputStream
   * @throws IOException wenn ein Fehler auftritt
   */
  public abstract OutputStream getOutputStream() throws IOException;

  /**
   * Die Implementierung dieser Methode muss einen InputStream liefern, aus dem
   * die gepufferten Daten gelesen werden knnen. Der erste Aufruf dieser
   * Methode beendet den Schreibvorgang in diesen Puffer.
   *
   * <b>Achtung:</b> Der zurckgegebene InputStream muss die reset()-Methode
   * in der Weise implementieren, dass nach deren Aufruf wieder von vorn ab dem
   * ersten Byte gelesen wird. Die markSupported()-Methode muss
   * <b>false</b> zurckliefern.
   * @return den InputStream
   * @throws IOException wenn ein Fehler auftritt
   * @see java.io.InputStream#reset()
   * @see java.io.InputStream#markSupported()
   */
  public abstract InputStream getInputStream() throws IOException;

  /**
   * Diese Methode muss die Anzahl der in den Puffer geschriebenen Bytes
   * zurckgeben.
   * @return Anzahl der Bytes
   */
  public abstract long getLength();

  /**
   * Liefert eine Versionsnummer.
   * @return Versionsnummer
   */
  public abstract String getVersion();

  /**
   * Liefert den Namen des Herstellers.
   * @return Herstellername
   */
  public abstract String getVendor();
}
