package de.osci.osci12.extinterfaces.crypto;

import de.osci.osci12.encryption.OSCICipherException;

import java.security.cert.X509Certificate;


/**
 * Diese abstrakte Klasse stellt die Schnittstelle der Bibliothek fr
 * die Anbindung von Entschlsselungs-Modulen (Crypto-Token) dar. Anwendungen, die
 * OSCI-Nachrichten entschlsseln wollen, mssen an das entschlsselnde Rollenobjekt
 * eine Implementation dieser Klasse bergeben.
 *
 * <p>Copyright  2010 Freie Hansestadt Bremen</p>
 * <p>Erstellt von bremen online services Entwicklungs- und
 * Betriebsgesellschaft GmbH & Co. KG</p>
 * <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
 * freie Softwarebibliotheken genutzt werden.</p>Die Lizenzbestimmungen knnen
 * unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
 * Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
 * 28015 Bremen schriftlich angefordert werden.</p>
 *
 *
 * @author R. Lindemann, N.Bngener
 * @version 1.5
 */
public abstract class Decrypter
{
  /**
   * Liefert die Versionsnummer
   * @return Versionsnummer
   */
  public abstract String getVersion();

  /**
   * Sollte den Namen des Herstellers zurckgeben.
   * @return Herstellername
   */
  public abstract String getVendor();

  /**
   * Die Implementierung dieser Methode muss das Verschlsselungszertifikat zurckgeben.
   * @return Verschlsselungszertifikat
   */
  public abstract X509Certificate getCertificate();

  //  public abstract byte[] decrypt(byte[] data, byte[] encryptedSessionKey);
  /**
   * Die Implementierung dieser Methode muss das bergebene Byte-Array entschlsseln.
   * @param data zu entschlsselndes Byte-Array
   * @return entschlsseltes Byte-Array
   * @throws OSCICipherException wenn beim Erstellen der Signatur ein Fehler auftritt
   * @throws de.osci.osci12.common.OSCICancelledException wenn der Vorgang vom Anwender abgebrochen wird
   */
  public abstract byte[] decrypt(byte[] data) throws OSCICipherException, 
                                                     de.osci.osci12.common.OSCICancelledException;
}
