package de.osci.osci12.messagetypes;

import de.osci.osci12.common.DialogHandler;
import de.osci.osci12.messageparts.MessagePartsFactory;
import de.osci.osci12.messageparts.ProcessCardBundleBuilder;
import de.osci.osci12.soapheader.ControlBlockHBuilder;
import de.osci.osci12.soapheader.DesiredLanguagesH;
import de.osci.osci12.soapheader.IntermediaryCertificatesHBuilder;
import de.osci.osci12.soapheader.NonIntermediaryCertificatesHBuilder;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;


/**
 * AcceptDelivery-Parser.
 *
 * <p>Copyright  2010 Freie Hansestadt Bremen</p>
 * <p>Erstellt von bremen online services Entwicklungs- und
 * Betriebsgesellschaft GmbH & Co. KG</p>
 * <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
 * freie Softwarebibliotheken genutzt werden.</p>Die Lizenzbestimmungen knnen
 * unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
 * Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
 * 28015 Bremen schriftlich angefordert werden.</p>
 *
 *
 * @author R. Lindemann / N. Bngener
 * @version 1.5
 */
class AcceptDeliveryBuilder extends OSCIMessageBuilder
{
  private static Log log = LogFactory.getLog(AcceptDeliveryBuilder.class);
  private ProcessCardBundleBuilder processCardBuilder = null;

  /**
   * Konstruktor fr den aufrufenden Mime-Parser
   *
   *@param  parentBuilder  Parent Object in den meisten Fllen der OSCIEnvelopeBuilder
   */
  public AcceptDeliveryBuilder(OSCIEnvelopeBuilder parentBuilder)
  {
    super(parentBuilder);
    msg = new de.osci.osci12.messagetypes.AcceptDelivery();
  }

  /**
   *   berschreibt die character Methode des DefaultHandler
   *
   *@param  uri                           Description of Parameter
   *@param  localName                     Description of Parameter
   *@param  qName                         Description of Parameter
   *@param  attributes                    Description of Parameter
   *@exception  SAXException  Description of Exception
   */
  public void startElement(String uri, String localName, String qName, Attributes attributes)
                    throws SAXException
  {
    if (log.isDebugEnabled())
      log.debug("Start-Element: " + localName);

    if (insideBody)
    {
      if (localName.equals("ContentPackage") && uri.equals(OSCI_XMLNS))
      {
        parentBuilder.xmlReader.setContentHandler(MessagePartsFactory.createContentPackageBuilder(this));
      }
      else
        throw new SAXException(DialogHandler.text.getString("unexpected_entry") + ": " + localName);
    }
    else if (insideHeader)
    {
      if (log.isDebugEnabled())
        log.debug("inside Header");

      // ### Auswerten des ControlBlock Headers###
      if (localName.equals("ControlBlock") && uri.equals(OSCI_XMLNS))
        parentBuilder.xmlReader.setContentHandler(new ControlBlockHBuilder(this, attributes, new int[] { 0, 1, 0, 0 }));
      // ### Auswerten des ClientSignature Headers###
      else if (localName.equals("ClientSignature") && uri.equals(OSCI_XMLNS))
        parentBuilder.xmlReader.setContentHandler(MessagePartsFactory.createOsciSignatureBuilder(parentBuilder.xmlReader,
                                                                                                 this, attributes));
      else if (localName.equals("DesiredLanguages") && uri.equals(OSCI_XMLNS))
      {
        msg.desiredLanguagesH = new DesiredLanguagesH(attributes.getValue("LanguagesList"));
        msg.desiredLanguagesH.setRefID(attributes.getValue("Id"));
        msg.desiredLanguagesH.setNSPrefixes(msg);
        msg.dialogHandler.setLanguageList(attributes.getValue("LanguagesList"));
      }
      else if (localName.equals("acceptDelivery") && uri.equals(OSCI_XMLNS))
        ;
      else if (localName.equals("ProcessCardBundle") && uri.equals(OSCI_XMLNS))
      {
        int[] check = { 1, 1, 0, -1 };
        processCardBuilder = new ProcessCardBundleBuilder("ProcessCardBundle", parentBuilder.xmlReader, this, check);
        parentBuilder.xmlReader.setContentHandler(processCardBuilder);
      }
      else if (localName.equals("IntermediaryCertificates") && uri.equals(OSCI_XMLNS))
      {
        int[] check = { -1, -1 };
        parentBuilder.xmlReader.setContentHandler(new IntermediaryCertificatesHBuilder(this, attributes, check));
      }
      else if (localName.equals("NonIntermediaryCertificates") && uri.equals(OSCI_XMLNS))
      {
        int[] check = { -1, -1, -1, -1, -1, -1, 0 };
        parentBuilder.xmlReader.setContentHandler(new NonIntermediaryCertificatesHBuilder(this, attributes, check));
      }
      else
        startCustomSoapHeader(uri, localName, qName, attributes);
    }
    else
      super.startElement(uri, localName, qName, attributes);
  }

  /**
   *  Description of the Method
   *
   *@param  uri                           Description of Parameter
   *@param  localName                     Description of Parameter
   *@param  qName                         Description of Parameter
   *@exception  SAXException  Description of Exception
   */
  public void endElement(String uri, String localName, String qName)
                  throws SAXException
  {
    if (log.isDebugEnabled())
      log.debug("End-Element: " + localName);

    if (localName.equals("Header") && uri.equals(SOAP_XMLNS))
    {
      ((AcceptDelivery) msg).setProcessCardBundle(processCardBuilder.getProcessCardBundleObject());
      insideHeader = false;
    }
    else
      super.endElement(uri, localName, qName);

    currentElement = null;
  }
}
