package de.osci.osci12.messagetypes;

import de.osci.osci12.messageparts.Content;
import de.osci.osci12.messageparts.ContentContainer;
import de.osci.osci12.messageparts.EncryptedDataOSCI;


/**
 * Dieses Interface wird von allen Nachrichtentypen implementiert, die
 * Inhaltsdaten enthalten.
 *
 * <p>Copyright  2010 Freie Hansestadt Bremen</p>
 * <p>Erstellt von bremen online services Entwicklungs- und
 * Betriebsgesellschaft GmbH & Co. KG</p>
 * <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
 * freie Softwarebibliotheken genutzt werden.</p>Die Lizenzbestimmungen knnen
 * unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
 * Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
 * 28015 Bremen schriftlich angefordert werden.</p>
 *
 *
 * @author R. Lindemann / N. Bngener
 * @version 1.5
 */
public interface ContentPackageI
{
  /**
   * Liefert die Message-ID der Nachricht (Antwort).
   * @return Message-ID
   * @since 1.0.3
   */
  public String getMessageId();

  /**
   * Liefert den im Antwortlaufzettel enthaltenen Betreff-Eintrag.
   * @return den Betreff der Antwortnachricht
   * @since 1.0.3
   */
  public String getSubject();

  /**
   * Liefert die in die Nachricht eingestellten (unverschlsselten) Inhaltsdaten als ContentContainer-Objekte.
   * @return enthaltene ContentContainer mit Inhaltsdaten
   * @see de.osci.osci12.messageparts.ContentContainer
   */
  public ContentContainer[] getContentContainer();

  /**
   * Durchsucht <b>die unverschlsselten</b> Inhaltsdaten nach dem ContentContainer
   * mit der bergebenen RefID.
   * @param refID zu suchende RefID
   * @return den zugehrigen ContentContainer oder null, wenn die Referenz
   * nicht gefunden wurde.
   */
  public ContentContainer getContentContainerByRefID(String refID);
  
  /**
   * Durchsucht <b>die unverschlsselten</b> ContentContainer nach dem Content
   * mit der bergebenen RefID.
   * @param refID zu suchende RefID
   * @return den zugehrigen Content oder null, wenn die Referenz
   * nicht gefunden wurde.
   */
  public Content getContentByRefID(String refID);

  /**
   * Liefert die in die Nachricht eingestellten verschlsselten Inhaltsdaten
   * als EncryptedData-Objekte.
   * @return enthaltene EncryptedData-Objekt mit verschlsselten Inhaltsdaten
   * @see EncryptedDataOSCI
   */
  public EncryptedDataOSCI[] getEncryptedData();
}
