package de.osci.osci12.messagetypes;

import de.osci.osci12.common.DialogHandler;
import de.osci.osci12.messageparts.MessagePartParser;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

import java.util.Vector;


/**
 * Feedback-Parser.
 *
 * <p>Copyright  2010 Freie Hansestadt Bremen</p>
 * <p>Erstellt von bremen online services Entwicklungs- und
 * Betriebsgesellschaft GmbH & Co. KG</p>
 * <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
 * freie Softwarebibliotheken genutzt werden.</p>Die Lizenzbestimmungen knnen
 * unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
 * Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
 * 28015 Bremen schriftlich angefordert werden.</p>
 *
 *
 * @author R. Lindemann / N. Bngener
 * @version 1.5
 */
class FeedbackBuilder extends MessagePartParser
{
  private static Log log = LogFactory.getLog(FeedbackBuilder.class);
  private Vector feedbacks = new Vector();
  private String[] entry;

  /**
   *  Constructor for the FeedbackBuilder object
   *
   *@param  parent  Description of Parameter
   */
  public FeedbackBuilder(de.osci.osci12.messagetypes.OSCIMessageBuilder parent)
  {
    super(parent);
  }

  /**
   *  Description of the Method
   *
   *@param  uri                           Description of Parameter
   *@param  localName                     Description of Parameter
   *@param  qName                         Description of Parameter
   *@param  attributes                    Description of Parameter
   *@exception  SAXException  Description of Exception
   */
  public void startElement(String uri, String localName, String qName, Attributes attributes)
                    throws SAXException
  {
    if (log.isDebugEnabled())
      log.debug("Start Element in Feedback: " + localName);

    if (localName.equals("Entry") && uri.equals(OSCI_XMLNS))
    {
      entry = new String[3];
      entry[0] = attributes.getValue("xml:lang");
    }
    else if (localName.equals("Code") && uri.equals(OSCI_XMLNS))
    {
      currentElement = new StringBuffer();
    }
    else if (localName.equals("Text") && uri.equals(OSCI_XMLNS))
    {
      currentElement = new StringBuffer();
    }
    else
    {
      throw new SAXException(DialogHandler.text.getString("unexpected_entry") + ": " + localName);
    }
  }

  /**
   *  Description of the Method
   *
   *@param  uri                           Description of Parameter
   *@param  localName                     Description of Parameter
   *@param  qName                         Description of Parameter
   *@exception  SAXException  Description of Exception
   */
  public void endElement(String uri, String localName, String qName)
                  throws SAXException
  {
    if (log.isDebugEnabled())
      log.debug("End Element in Feedback: " + localName);

    if (localName.equals("Feedback") && uri.equals(OSCI_XMLNS))
    {
      ((OSCIResponseTo) msg).feedBack = feedbacks;
      xmlReader.setContentHandler(parentHandler);
    }
    else if (localName.equals("Entry") && uri.equals(OSCI_XMLNS))
    {
      feedbacks.add(entry);
    }
    else if (localName.equals("Code") && uri.equals(OSCI_XMLNS))
    {
      entry[1] = currentElement.toString();
    }
    else if (localName.equals("Text") && uri.equals(OSCI_XMLNS))
    {
      entry[2] = currentElement.toString();
    }
    else
    {
      throw new SAXException(DialogHandler.text.getString("unexpected_entry") + ": " + localName);
    }

    currentElement = null;
  }
}
