package de.osci.osci12.messagetypes;

import de.osci.helper.CanParser;
import de.osci.helper.Canonizer;
import de.osci.osci12.common.DialogHandler;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.XMLReader;
import org.xml.sax.helpers.DefaultHandler;

import java.util.Vector;


/**
 * Superklasse der Nachrichtenparser. Wird vom Anwender nicht bentigt.
 *
 * <p>Copyright  2010 Freie Hansestadt Bremen</p>
 * <p>Erstellt von bremen online services Entwicklungs- und
 * Betriebsgesellschaft GmbH & Co. KG</p>
 * <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
 * freie Softwarebibliotheken genutzt werden.</p>Die Lizenzbestimmungen knnen
 * unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
 * Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
 * 28015 Bremen schriftlich angefordert werden.</p>
 *
 *
 * @author R. Lindemann, N.Bngener
 * @version 1.5
 */
public class OSCIMessageBuilder extends DefaultHandler
{
  public final static String SOAP_XMLNS = "http://schemas.xmlsoap.org/soap/envelope/";
  public final static String OSCI_XMLNS = "http://www.osci.de/2002/04/osci";
  public final static String DS_XMLNS = "http://www.w3.org/2000/09/xmldsig#";
  public final static String XENC_XMLNS = "http://www.w3.org/2001/04/xmlenc#";
  public final static String XSI_XMLNS = "http://www.w3.org/2001/XMLSchema-instance";
  private static Log log = LogFactory.getLog(OSCIMessageBuilder.class);
  protected StringBuffer currentElement = null;
  /**
   *  Description of the Field
   */
  protected OSCIMessage msg = null;
  protected boolean insideHeader = false;
  protected boolean insideBody = false;
  public OSCIEnvelopeBuilder parentBuilder = null;
  protected Vector customSoapHeader = new Vector();

  /**
   * Creates a new OSCIMessageBuilder object.
   *
   * @param eb undocumented
   */
  public OSCIMessageBuilder(OSCIEnvelopeBuilder eb)
  {
    this.parentBuilder = eb;
  }

  /**
   * undocumented
   *
   * @return undocumented
   */
  public OSCIMessage getOSCIMessage()
  {
    return msg;
  }

  /**
   * undocumented
   *
   * @return undocumented
   */
  public XMLReader getXMLReader()
  {
    return parentBuilder.xmlReader;
  }

  /**
   * undocumented
   *
   * @param uri undocumented
   * @param localName undocumented
   * @param qName undocumented
   * @param attributes undocumented
   *
   * @throws SAXException undocumented
   */
  public void startElement(String uri, String localName, String qName, Attributes attributes)
                    throws SAXException
  {
    log.debug("ENV-Builder startElement: " + localName + " " + uri.length());

    if (localName.equals("Header") && uri.equals(SOAP_XMLNS))
    {
      insideHeader = true;
    }
    else if (localName.equals("Body") && uri.equals(SOAP_XMLNS))
    {
      insideBody = true;
      //      msg.bodyId = attributes.getValue("Id");
    }
    else if (!(localName.equals("Envelope") && uri.equals(SOAP_XMLNS)))
      throw new SAXException(DialogHandler.text.getString("unexpected_entry") + ": " + localName);
  }

  /**
   * undocumented
   *
   * @param uri undocumented
   * @param localName undocumented
   * @param qName undocumented
   * @param attributes undocumented
   *
   * @throws SAXException undocumented
   */
  protected void startCustomSoapHeader(String uri, String localName, String qName, Attributes attributes)
                                throws SAXException
  {
    try
    {
      CanParser cp = new CanParser(customSoapHeader, parentBuilder.xmlReader,
                                   parentBuilder.xmlReader.getContentHandler(), qName);
      parentBuilder.xmlReader.setContentHandler(cp);
      cp.startDocument();
      cp.startElement(uri, localName, qName, attributes);
    }
    catch (SAXException ex)
    {
      throw ex;
    }
    catch (Exception ex)
    {
      throw new SAXException(DialogHandler.text.getString("sax_exception_customheader"), ex);
    }
  }

  /**
   * undocumented
   *
   * @param uri undocumented
   * @param localName undocumented
   * @param qName undocumented
   *
   * @throws SAXException undocumented
   */
  public void endElement(String uri, String localName, String qName)
                  throws SAXException
  {
    if (localName.equals("Header") && uri.equals(SOAP_XMLNS))
      insideHeader = false;
    else if (localName.equals("Body") && uri.equals(SOAP_XMLNS))
      insideBody = false;
    else if (localName.equals("Envelope") && uri.equals(SOAP_XMLNS))
    {
      parentBuilder.xmlReader.setContentHandler(parentBuilder);

      for (int i = 0; i < customSoapHeader.size(); i++)
        msg.addCustomHeader((String) customSoapHeader.get(i));
    }

    currentElement = null;
  }

  /**
   * undocumented
   *
   * @param ch undocumented
   * @param start undocumented
   * @param length undocumented
   *
   * @throws SAXException undocumented
   */
  public void characters(char[] ch, int start, int length)
                  throws SAXException
  {
    if (currentElement == null)
    {
      for (int i = 0; i < length; i++)
      {
        if (ch[start + i] > ' ')
          throw new SAXException(DialogHandler.text.getString("unexpected_char"));
      }
    }
    else
      currentElement.append(ch, start, length);
  }

  /**
   * undocumented
   *
   * @return undocumented
   */
  public Canonizer getCanStream()
  {
    return parentBuilder.hashNCanStream;
  }
}
