/* Generated by Together */
package de.osci.osci12.messagetypes;

import de.osci.helper.Tools;
import de.osci.osci12.OSCIException;
import de.osci.osci12.common.Constants;
import de.osci.osci12.common.DialogHandler;
import de.osci.osci12.messageparts.Attachment;
import de.osci.osci12.messageparts.Body;
import de.osci.osci12.messageparts.Content;
import de.osci.osci12.messageparts.ContentContainer;
import de.osci.osci12.messageparts.EncryptedDataOSCI;
import de.osci.osci12.messageparts.Inspection;
import de.osci.osci12.messageparts.ProcessCardBundle;
import de.osci.osci12.messageparts.Timestamp;
import de.osci.osci12.roles.Addressee;
import de.osci.osci12.roles.Intermed;
import de.osci.osci12.roles.OSCIRoleException;
import de.osci.osci12.soapheader.DesiredLanguagesH;
import de.osci.osci12.soapheader.IntermediaryCertificatesH;
import de.osci.osci12.soapheader.OsciH;

import java.io.IOException;

import java.security.NoSuchAlgorithmException;


/**
 * <p>Diese Klasse reprsentiert Nachrichtenobjekte fr Bearbeitungsauftrge.
 * Der Intermedir erzeugt nach dem Erhalt eines Abwicklungsauftrags eine
 * Instanz dieser Klasse und sendet die Nachricht an den Empfnger (hier als Supplier) .
 * Als Antwort auf diese Nachricht muss der Empfnger ein
 * ResponseToProcessDelivery-Nachrichtenobjekt mit einer Rckmeldung (Feedback)
 * aufbauen und an den Intermedir zurcksenden.</p>
 *
 * <p>Copyright  2010 Freie Hansestadt Bremen</p>
 * <p>Erstellt von bremen online services Entwicklungs- und
 * Betriebsgesellschaft GmbH & Co. KG</p>
 * <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
 * freie Softwarebibliotheken genutzt werden.</p>Die Lizenzbestimmungen knnen
 * unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
 * Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
 * 28015 Bremen schriftlich angefordert werden.</p>
 *
 *
 * @author R. Lindemann, N. Bngener
 * @version 1.5
 *
 * @see de.osci.osci12.messagetypes.ResponseToProcessDelivery
 */
public class ProcessDelivery extends OSCIRequest implements ContentPackageI
{
  //  private static Log log = LogFactory.getLog(ProcessDelivery.class);
  ProcessCardBundle processCardBundle;

  ProcessDelivery(MediateDelivery md) throws OSCIRoleException, 
                                             NoSuchAlgorithmException
  {
    super(new DialogHandler((Intermed) md.dialogHandler.getSupplier(), md.addressee,
                            (de.osci.osci12.extinterfaces.TransportI) null));
    messageType = PROCESS_DELIVERY;
    uriReceiver = md.uriReceiver;
    addressee = md.addressee;
    originator = md.originator;
    desiredLanguagesH = new DesiredLanguagesH(md.desiredLanguagesH.getLanguageList());

    ContentContainer[] con = md.getContentContainer();

    for (int i = 0; i < con.length; i++)
      addContentContainer(con[i]);

    EncryptedDataOSCI[] enc = md.getEncryptedData();

    for (int i = 0; i < enc.length; i++)
      addEncryptedData(enc[i]);

    Attachment[] att = md.getAttachments();

    for (int i = 0; i < att.length; i++)
      addAttachment(att[i]);

    for (int i = 0; i < md.getOtherAuthors().length; i++)
      otherAutors.put(md.getOtherAuthors()[i].id, md.getOtherAuthors()[i]);

    for (int i = 0; i < md.getOtherReaders().length; i++)
      otherReaders.put(md.getOtherReaders()[i].id, md.getOtherReaders()[i]);

    dialogHandler.setEncryption(md.getDialogHandler().isEncryption());
    dialogHandler.getControlblock().setResponse(null);
    dialogHandler.getControlblock().setChallenge(Tools.createRandom(10));
    dialogHandler.getControlblock().setConversationID(null);
    dialogHandler.getControlblock().setSequenceNumber(-1);
  }

  ProcessDelivery(DialogHandler dh)
  {
    super(dh);
    addressee = ((Addressee) dh.getSupplier());
    messageType = PROCESS_DELIVERY;
  }

  /**
   * Creates a new ProcessDelivery object.
   */
  public ProcessDelivery()
  {
    messageType = PROCESS_DELIVERY;
  }

  /**
   * Diese Methode liefert den Laufzettel der Zustellung zurck oder null,
   * wenn bei der Verarbeitung der Nachricht ein Fehler aufgetreten ist.
   * Die Informationen im Laufzettel knnen auch direkt ber die einzelnen
   * getX()-Methoden ausgewertet werden.
   *
   * @return den Laufzettel als ProcessCardBundle-Objekt, im Fehlerfall null
   * @see #getTimestampCreation()
   * @see #getTimestampForwarding()
   * @see #getInspections()
   * @see #getSubject()
   * @see #getRecentModification()
   * @see #getMessageId()
   */
  public ProcessCardBundle getProcessCardBundle()
  {
    return processCardBundle;
  }

  /**
   * Liefert die in die Nachricht eingestellten (unverschlsselten) Inhaltsdaten als ContentContainer-Objekte.
   * @return enthaltene ContentContainer mit Inhaltsdaten
   * @see de.osci.osci12.messageparts.ContentContainer
   */
  public ContentContainer[] getContentContainer()
  {
    return super.getContentContainer();
  }

  /**
   * Durchsucht <b>die unverschlsselten</b> Inhaltsdaten nach dem ContentContainer
   * mit der bergebenen RefID.
   * @param refID zu suchende RefID
   * @return den zugehrigen ContentContainer oder null, wenn die Referenz
   * nicht gefunden wurde.
   */
  public ContentContainer getContentContainerByRefID(String refID)
  {
	  return super.getContentContainerByRefID(refID);
  }
  
  /**
   * Durchsucht <b>die unverschlsselten</b> ContentContainer nach dem Content
   * mit der bergebenen RefID.
   * @param refID zu suchende RefID
   * @return den zugehrigen Content oder null, wenn die Referenz
   * nicht gefunden wurde.
   */
  public Content getContentByRefID(String refID)
  {
	return super.getContentByRefID(refID);  
  }

  /**
   * Liefert die in die Nachricht eingestellten verschlsselten Inhaltsdaten
   * als EncryptedData-Objekte.
   * @return enthaltene EncryptedData-Objekt mit verschlsselten Inhaltsdaten
   * @see EncryptedDataOSCI
   */
  public EncryptedDataOSCI[] getEncryptedData()
  {
    return super.getEncryptedData();
  }

  /**
   * Liefert den im Laufzettel enthaltenen Zeitstempel vom Zeitpunkt des Eingangs
   * des Abwicklungsauftrags beim Intermedir (ggf. null).
   * @return Zeitstempel der Einreichung beim Intermedir
   * @see #getProcessCardBundle()
   */
  public Timestamp getTimestampCreation()
  {
    if (processCardBundle != null)
      return processCardBundle.getCreation();
    else

      return null;
  }

  /**
   * Liefert den im Laufzettel enthaltenen Zeitstempel vom Zeitpunkt des
   * vollstndigen Aufbaus des Bearbeitungsauftrags vom Intermedir fr den Empfnger (ggf. null).
   * @return Zeitstempel der Erstellung des Bearbeitungsauftrags durch den Intermedir
   * @see #getProcessCardBundle()
   */
  public Timestamp getTimestampForwarding()
  {
    if (processCardBundle != null)
      return processCardBundle.getForwarding();
    else

      return null;
  }

  /**
   * Liefert die Ergebnisse der Zertifikatsprfungen in Form von Inspection-Objekten,
   * die im ProcessCardBundle-Objekt enthalten sind (ggf. null).
   * @see #getProcessCardBundle()
   */
  public Inspection[] getInspections()
  {
    if (processCardBundle != null)
      return processCardBundle.getInspections();
    else

      return null;
  }

  //  void setInspections(Inspection[] inspections) {this.inspections = inspections;}
  /**
   * Liefert den im Laufzettel enthaltenen Betreff-Eintrag (ggf. null).
   * @return den Betreff der Zustellung
   * @see #getProcessCardBundle()
   */
  public String getSubject()
  {
    if (processCardBundle != null)
      return processCardBundle.getSubject();
    else

      return null;
  }

  /**
   * Liefert das Datum der letzten nderung des Laufzettels (ggf. null). Das Format
   * entspricht dem XML-Schema nach http://www.w3.org/TR/xmlschema-2/#dateTime
   * @return Datum der letzten nderung
   * @see #getProcessCardBundle()
   */
  public String getRecentModification()
  {
    if (processCardBundle != null)
      return processCardBundle.getRecentModification();
    else

      return null;
  }

  /**
   * Liefert die Message-ID der Nachricht.
   * @return Message-ID
   * @see #getProcessCardBundle()
   */
  public String getMessageId()
  {
    if (processCardBundle != null)
      return processCardBundle.getMessageId();
    else

      return null;
  }

  /**
   * Liefert die vom Intermedir mitgesendete Message-ID fr die Antwortnachricht.
   * @return Message-ID fr Antwortnachricht oder null, wenn keine Protokollierung
   * verlangt ist.
   */
  public String getMessageIdResponse()
  {
    return messageId;
  }

  void sign()
     throws IOException, 
            OSCIException, 
            de.osci.osci12.common.OSCICancelledException, 
            java.security.NoSuchAlgorithmException
  {
    super.sign();
    // Der IntemedCertHeader liegt hier weiter unten
    messageParts.set(5, intermediaryCertificatesH);
    messageParts.set(3, null);
  }

  /**
   * undocumented
   *
   * @throws OSCIRoleException undocumented
   * @throws IOException undocumented
   */
  protected void compose() throws OSCIException, 
                                  IOException, 
                                  NoSuchAlgorithmException
  {
    super.compose();
    messageParts.set(3, null);

    if (processCardBundle != null)
    {
      String pdh = "<" + osciNSPrefix + ":MessageIdResponse>" +
                   de.osci.helper.Base64.encode(messageId.getBytes(Constants.CHAR_ENCODING)) + "</" + osciNSPrefix +
                   ":MessageIdResponse>" + processCardBundle.writeToString();
      osciH = new OsciH("processDelivery", "" + pdh);
    }
    else
      osciH = new OsciH("processDelivery", "");

    messageParts.add(osciH);

    if (dialogHandler.getClient().hasCipherCertificate())
    {
      if (intermediaryCertificatesH == null)
        intermediaryCertificatesH = new IntermediaryCertificatesH();

      intermediaryCertificatesH.setCipherCertificateIntermediary((Intermed) dialogHandler.getClient());
    }

    messageParts.add(intermediaryCertificatesH);
    createNonIntermediaryCertificatesH();
    messageParts.add(nonIntermediaryCertificatesH);
    messageParts.addAll(customHeaders);
    body = new Body(getContentContainer(), getEncryptedData());
    messageParts.add(body);
    stateOfMsg |= STATE_COMPOSED;
  }
}
