package de.osci.osci12.messagetypes;

import de.osci.osci12.OSCIException;
import de.osci.osci12.common.DialogHandler;
import de.osci.osci12.messageparts.Body;
import de.osci.osci12.roles.OSCIRoleException;
import de.osci.osci12.roles.Originator;

import java.io.IOException;

import java.security.NoSuchAlgorithmException;


/**
 * <p>Diese Klasse reprsentiert die Antwortnachricht auf die Beendigung
 * eines expliziten Dialogs.
 * Clients erhalten vom Intermedir eine Instanz dieser Klasse als Antwort auf
 * ein an den Intermedir gesendetes ExitDialog-Objekt.</p>
 * <p>Copyright  2010 Freie Hansestadt Bremen</p>
 * <p>Erstellt von bremen online services Entwicklungs- und
 * Betriebsgesellschaft GmbH & Co. KG</p>
 * <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
 * freie Softwarebibliotheken genutzt werden.</p>Die Lizenzbestimmungen knnen
 * unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
 * Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
 * 28015 Bremen schriftlich angefordert werden.</p>
 *
 *
 * @author R. Lindemann, N. Bngener
 * @version 1.5
 * @see de.osci.osci12.messagetypes.ExitDialog
 * @see de.osci.osci12.common.DialogHandler
 */
public class ResponseToExitDialog extends OSCIResponseTo
{
  //  private static Log log = LogFactory.getLog(ResponseToExitDialog.class);
  ResponseToExitDialog(DialogHandler dh)
  {
    super(dh);
    messageType = RESPONSE_TO_EXIT_DIALOG;
    originator = ((Originator) dh.getClient());
    // Challenge wird in compose() zu null gesetzt.
    dialogHandler.getControlblock().setResponse(dialogHandler.prevChallenge);
  }

  ResponseToExitDialog()
  {
    super();
    messageType = RESPONSE_TO_EXIT_DIALOG;
    dialogHandler.explicitDialog = false;
  }

  /**
   * undocumented
   *
   * @throws OSCIRoleException undocumented
   * @throws IllegalStateException undocumented
   */
  protected void compose() throws OSCIException, 
                                  NoSuchAlgorithmException, 
                                  IOException
  {
    super.compose();
    messageParts.addAll(customHeaders);

    if (dialogHandler.getControlblock().getResponse() == null)
      throw new IllegalStateException(DialogHandler.text.getString("missing_entry") + ": Response");

    if (dialogHandler.getControlblock().getConversationID() == null)
      throw new IllegalStateException(DialogHandler.text.getString("missing_entry") + ": Conversation-Id");

    if (dialogHandler.getControlblock().getSequenceNumber() == -1)
      throw new IllegalStateException(DialogHandler.text.getString("missing_entry") + ": SequenceNumber");

    dialogHandler.getControlblock().setChallenge(null);
    dialogHandler.explicitDialog = false;

    if (feedBack == null)
      throw new IllegalStateException(DialogHandler.text.getString("missing_entry") + ": Feedback");

    body = new Body("<" + osciNSPrefix + ":responseToExitDialog>" + writeFeedBack() + "</" + osciNSPrefix +
                    ":responseToExitDialog>");
    body.setNSPrefixes(this);
    messageParts.add(body);
    stateOfMsg |= STATE_COMPOSED;
  }
}
