package de.osci.osci12.messagetypes;

import de.osci.osci12.common.Constants;
import de.osci.osci12.messageparts.MessagePartsFactory;
import de.osci.osci12.messageparts.ProcessCardBundleBuilder;
import de.osci.osci12.soapheader.ControlBlockHBuilder;
import de.osci.osci12.soapheader.IntermediaryCertificatesHBuilder;
import de.osci.osci12.soapheader.NonIntermediaryCertificatesHBuilder;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

import java.io.IOException;


/**
 * ResponseToFetchDelivery-Parser.
 *
 * <p>Copyright  2010 Freie Hansestadt Bremen</p>
 * <p>Erstellt von bremen online services Entwicklungs- und
 * Betriebsgesellschaft GmbH & Co. KG</p>
 * <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
 * freie Softwarebibliotheken genutzt werden.</p>Die Lizenzbestimmungen knnen
 * unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
 * Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
 * 28015 Bremen schriftlich angefordert werden.</p>
 *
 *
 * @author R. Lindemann / N. Bngener
 * @version 1.5
 */
class ResponseToFetchDeliveryBuilder extends OSCIMessageBuilder
{
  private static Log log = LogFactory.getLog(ResponseToFetchDeliveryBuilder.class);
  /**
   *  Objekt ProcessCardBundle fr ProcessCard Information
   */
  private ProcessCardBundleBuilder processCardBuilder = null;

  /**
   *  Constructor for the ResponseToForwardDeliveryBuilder object
   *
   *@param  parentHandler  Description of Parameter
   */
  public ResponseToFetchDeliveryBuilder(OSCIEnvelopeBuilder parentHandler)
  {
    super(parentHandler);
    msg = new de.osci.osci12.messagetypes.ResponseToFetchDelivery(parentHandler.dhHandler);
  }

  /**
   *  Description of the Method
   *
   *@param  uri                           Description of Parameter
   *@param  localName                     Description of Parameter
   *@param  qName                         Description of Parameter
   *@param  attributes                    Description of Parameter
   *@exception  SAXException  Description of Exception
   */
  public void startElement(String uri, String localName, String qName, Attributes attributes)
                    throws SAXException
  {
    if (log.isDebugEnabled())
      log.debug("Start Element RspFwdDel: " + localName);

    if (insideBody)
    {
      if (localName.equals("ContentPackage") && uri.equals(OSCI_XMLNS))
        parentBuilder.xmlReader.setContentHandler(MessagePartsFactory.createContentPackageBuilder(this));
    }
    else if (insideHeader)
    {
      // ### Auswerten des ControlBlock Headers###
      if (localName.equals("ControlBlock") && uri.equals(OSCI_XMLNS))
        parentBuilder.xmlReader.setContentHandler(new ControlBlockHBuilder(this, attributes, new int[] { 1, 1, 1, 1 }));
      // ### Auswerten des ClientSignature Headers###
      else if (localName.equals("SupplierSignature") && uri.equals(OSCI_XMLNS))
        parentBuilder.xmlReader.setContentHandler(MessagePartsFactory.createOsciSignatureBuilder(parentBuilder.xmlReader,
                                                                                                 this, attributes));
      else if (localName.equals("responseToFetchDelivery") && uri.equals(OSCI_XMLNS))
      {
        ;
        //nothing to do
      }
      else if (localName.equals("ProcessCardBundle") && uri.equals(OSCI_XMLNS))
      {
        int[] check = { 1, -1, -1, -1 };
        processCardBuilder = new ProcessCardBundleBuilder("ProcessCardBundle", parentBuilder.xmlReader, this, check);
        parentBuilder.xmlReader.setContentHandler(processCardBuilder);
      }
      else if (localName.equals("Feedback") && uri.equals(OSCI_XMLNS))
      {
        parentBuilder.xmlReader.setContentHandler(new FeedbackBuilder(this));
      }
      else if (localName.equals("MessageId") && uri.equals(OSCI_XMLNS))
      {
        ((ResponseToFetchDelivery) msg).setSelectionMode(OSCIMessage.SELECT_BY_MESSAGE_ID);
        currentElement = new StringBuffer();
      }
      else if (localName.equals("ReceptionOfDelivery") && uri.equals(OSCI_XMLNS))
      {
        ((ResponseToFetchDelivery) msg).setSelectionMode(OSCIMessage.SELECT_BY_DATE_OF_RECEPTION);
        currentElement = new StringBuffer();
      }
      else if (localName.equals("RecentModification") && uri.equals(OSCI_XMLNS))
      {
        ((ResponseToFetchDelivery) msg).setSelectionMode(OSCIMessage.SELECT_BY_RECENT_MODIFICATION);
        currentElement = new StringBuffer();
      }
      else if (localName.equals("IntermediaryCertificates") && uri.equals(OSCI_XMLNS))
      {
        int[] check = { 0, -1 };
        parentBuilder.xmlReader.setContentHandler(new IntermediaryCertificatesHBuilder(this, attributes, check));
      }
      else if (localName.equals("NonIntermediaryCertificates") && uri.equals(OSCI_XMLNS))
      {
        int[] check = { -1, -1, -1, -1, -1, -1, 0 };
        parentBuilder.xmlReader.setContentHandler(new NonIntermediaryCertificatesHBuilder(this, attributes, check));
      }
      else if (!(localName.equals("fetchDelivery") && uri.equals(OSCI_XMLNS)) &&
                   !(localName.equals("SelectionRule") && uri.equals(OSCI_XMLNS)))
        startCustomSoapHeader(uri, localName, qName, attributes);
    }
    else
      super.startElement(uri, localName, qName, attributes);
  }

  /**
   *  Description of the Method
   *
   *@param  uri                           Description of Parameter
   *@param  localName                     Description of Parameter
   *@param  qName                         Description of Parameter
   *@exception  SAXException  Description of Exception
   */
  public void endElement(String uri, String localName, String qName)
                  throws SAXException
  {
    if (log.isDebugEnabled())
      log.debug("End-Element: " + localName);

    if (localName.equals("MessageId") && uri.equals(OSCI_XMLNS))
    {
      try
      {
        ((ResponseToFetchDelivery) msg).setSelectionRule(new String(de.osci.helper.Base64.decode(currentElement.toString()),
                                                                    Constants.CHAR_ENCODING));
      }
      catch (IOException ex)
      {
        throw new SAXException(ex);
      }
    }
    else if ((localName.equals("ReceptionOfDelivery") && uri.equals(OSCI_XMLNS)) ||
                 (localName.equals("RecentModification") && uri.equals(OSCI_XMLNS)))
    {
      ((ResponseToFetchDelivery) msg).setSelectionRule(currentElement.toString());
    }
    else if (localName.equals("Header") && uri.equals(SOAP_XMLNS))
    {
      ResponseToFetchDelivery rtmd = ((ResponseToFetchDelivery) msg);

      if (processCardBuilder != null)
      {
        rtmd.processCardBundle = processCardBuilder.getProcessCardBundleObject();
        rtmd.messageId = processCardBuilder.getProcessCardBundleObject().getMessageId();
      }

      insideHeader = false;
    }
    else
      super.endElement(uri, localName, qName);

    currentElement = null;
  }
}
