package de.osci.osci12.messagetypes;

import de.osci.helper.Tools;
import de.osci.osci12.OSCIException;
import de.osci.osci12.common.Constants;
import de.osci.osci12.common.DialogHandler;
import de.osci.osci12.messageparts.Body;
import de.osci.osci12.roles.OSCIRoleException;
import de.osci.osci12.roles.Originator;

import java.io.IOException;

import java.security.NoSuchAlgorithmException;


/**
 * <p>Dieses Klasse reprsentiert die Antwort des Intermedirs auf die
 * Anforderung einer Message-ID.
 * Clients erhalten vom Intermedir eine Instanz dieser Klasse, die eine Rckmeldung
 * ber den Erfolg der Operation (getFeedback()) sowie ggf. die angeforderte
 * Message-Id.</p>
 * <p>Copyright  2010 Freie Hansestadt Bremen</p>
 * <p>Erstellt von bremen online services Entwicklungs- und
 * Betriebsgesellschaft GmbH & Co. KG</p>
 * <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
 * freie Softwarebibliotheken genutzt werden.</p>Die Lizenzbestimmungen knnen
 * unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
 * Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
 * 28015 Bremen schriftlich angefordert werden.</p>
 *
 *
 * @author R. Lindemann, N. Bngener
 * @version 1.5
 * @see de.osci.osci12.messagetypes.GetMessageId
 */
public class ResponseToGetMessageId extends OSCIResponseTo
{
  //  private static Log log = LogFactory.getLog(ResponseToGetMessageId.class);
  ResponseToGetMessageId(DialogHandler dh, boolean parser)
                  throws NoSuchAlgorithmException
  {
    super(dh);
    messageType = RESPONSE_TO_GET_MESSAGE_ID;
    originator = ((Originator) dh.getClient());

    if (!parser)
    {
      dialogHandler.getControlblock().setResponse(dialogHandler.prevChallenge);
      dialogHandler.getControlblock().setChallenge(Tools.createRandom(10));
    }
  }

  ResponseToGetMessageId(DialogHandler dh) throws NoSuchAlgorithmException
  {
    this(dh, false);
  }

  /**
   * Liefert die angeforderte Message-ID. Wenn der Auftrag nicht
   * ordnungsgem abgewickelt wurde, wird <b>null</b> zurckgegeben.
   * In disem Fall sollte der Feedback-Eintrag ausgewertet werden.
   * @return Message-ID bzw. null im Fehlerfall
   * @see #getFeedback()
   */
  public String getMessageId()
  {
    return messageId;
  }

  void setMessageId(String messageId)
  {
    this.messageId = messageId;
  }

  /**
   * undocumented
   *
   * @throws OSCIRoleException undocumented
   * @throws IOException undocumented
   * @throws IllegalStateException undocumented
   */
  protected void compose() throws OSCIException, 
                                  IOException, 
                                  NoSuchAlgorithmException
  {
    super.compose();
    messageParts.addAll(customHeaders);

    if (dialogHandler.getControlblock().getSequenceNumber() == -1)
      throw new IllegalStateException(DialogHandler.text.getString("missing_entry") + ": SequenceNumber");

    if (feedBack == null)
      throw new IllegalStateException(DialogHandler.text.getString("missing_entry") + ": Feedback");

    StringBuffer bd = new StringBuffer("<");
    bd.append(osciNSPrefix);
    bd.append(":responseToGetMessageId>");
    bd.append(writeFeedBack());

    if ((messageId != null) && (messageId.length() > 0))
    {
      bd.append("<");
      bd.append(osciNSPrefix);
      bd.append(":MessageId>");
      bd.append(de.osci.helper.Base64.encode(messageId.getBytes(Constants.CHAR_ENCODING)));
      bd.append("</");
      bd.append(osciNSPrefix);
      bd.append(":MessageId>");
    }

    bd.append("</");
    bd.append(osciNSPrefix);
    bd.append(":responseToGetMessageId>");
    body = new Body(bd.toString());
    body.setNSPrefixes(this);
    messageParts.add(body);
    stateOfMsg |= STATE_COMPOSED;
  }
}
