package de.osci.osci12.messagetypes;

import de.osci.osci12.messageparts.MessagePartsFactory;
import de.osci.osci12.messageparts.ProcessCardBundleBuilder;
import de.osci.osci12.soapheader.ControlBlockHBuilder;
import de.osci.osci12.soapheader.IntermediaryCertificatesHBuilder;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

import java.security.NoSuchAlgorithmException;


/**
 * ResponseToStoreDelivery-Parser.
 *
 * <p>Copyright  2010 Freie Hansestadt Bremen</p>
 * <p>Erstellt von bremen online services Entwicklungs- und
 * Betriebsgesellschaft GmbH & Co. KG</p>
 * <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
 * freie Softwarebibliotheken genutzt werden.</p>Die Lizenzbestimmungen knnen
 * unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
 * Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
 * 28015 Bremen schriftlich angefordert werden.</p>
 *
 *
 * @author R. Lindemann / N. Bngener
 * @version 1.5
 */
class ResponseToStoreDeliveryBuilder extends OSCIMessageBuilder
{
  private static Log log = LogFactory.getLog(ResponseToStoreDelivery.class);
  /**
   *  Objekt ProcessCardBundle fr ProcessCard Information
   */
  private ProcessCardBundleBuilder processCardBuilder = null;

  /**
   *  Constructor for the ResponseToForwardDeliveryBuilder object
   *
   *@param  parentHandler  Description of Parameter
   */
  public ResponseToStoreDeliveryBuilder(OSCIEnvelopeBuilder parentHandler)
                                 throws NoSuchAlgorithmException
  {
    super(parentHandler);
    msg = new de.osci.osci12.messagetypes.ResponseToStoreDelivery(parentHandler.dhHandler, true);
  }

  /**
   *  Description of the Method
   *
   *@param  uri                           Description of Parameter
   *@param  localName                     Description of Parameter
   *@param  qName                         Description of Parameter
   *@param  attributes                    Description of Parameter
   *@exception  SAXException  Description of Exception
   */
  public void startElement(String uri, String localName, String qName, Attributes attributes)
                    throws SAXException
  {
    if (log.isDebugEnabled())
      log.debug("Start Element RspStrDel: " + localName);

    if (insideBody)
    {
    }
    else if (insideHeader)
    {
      // ### Auswerten des ControlBlock Headers###
      if (localName.equals("ControlBlock") && uri.equals(OSCI_XMLNS))
      {
        parentBuilder.xmlReader.setContentHandler(new ControlBlockHBuilder(this, attributes, //                        new int[]{1, -1, 1, 1}));
                                                                           //gendert, s. ControlBlockH
        new int[] { 1, -1, 1, 1 }));
      }

      // ### Auswerten des ClientSignature Headers###
      else if (localName.equals("SupplierSignature") && uri.equals(OSCI_XMLNS))
      {
        parentBuilder.xmlReader.setContentHandler(MessagePartsFactory.createOsciSignatureBuilder(parentBuilder.xmlReader,
                                                                                                 this, attributes));
      }
      else if (localName.equals("responseToStoreDelivery") && uri.equals(OSCI_XMLNS))
      {
        ;
        //nothing to do
      }
      else if (localName.equals("ProcessCardBundle") && uri.equals(OSCI_XMLNS))
      {
        int[] check = { 1, -1, -1, -1 };
        processCardBuilder = new ProcessCardBundleBuilder("ProcessCardBundle", parentBuilder.xmlReader, this, check);
        parentBuilder.xmlReader.setContentHandler(processCardBuilder);
      }
      else if (localName.equals("Feedback") && uri.equals(OSCI_XMLNS))
      {
        parentBuilder.xmlReader.setContentHandler(new FeedbackBuilder(this));
      }
      else if (localName.equals("IntermediaryCertificates") && uri.equals(OSCI_XMLNS))
      {
        int[] check = { 0, -1 };
        parentBuilder.xmlReader.setContentHandler(new IntermediaryCertificatesHBuilder(this, attributes, check));
      }
      else
        startCustomSoapHeader(uri, localName, qName, attributes);
    }
    else
      super.startElement(uri, localName, qName, attributes);
  }

  /**
   *  Description of the Method
   *
   *@param  uri                           Description of Parameter
   *@param  localName                     Description of Parameter
   *@param  qName                         Description of Parameter
   *@exception  SAXException  Description of Exception
   */
  public void endElement(String uri, String localName, String qName)
                  throws SAXException
  {
    if (log.isDebugEnabled())
      log.debug("End-Element: " + localName);

    if (localName.equals("Header"))
    {
      if (processCardBuilder != null)
        ((ResponseToStoreDelivery) msg).processCardBundle = processCardBuilder.getProcessCardBundleObject();

      insideHeader = false;
    }
    else
      super.endElement(uri, localName, qName);

    currentElement = null;
  }
}
