package de.osci.osci12.messagetypes;

import de.osci.osci12.common.Constants;
import de.osci.osci12.common.DialogHandler;

import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;


/**
 * <p>Dieses Klasse reprsentiert eine SOAP-Fehlermeldung auf Nachrichtenebene.</p>
 * <p>Copyright  2010 Freie Hansestadt Bremen</p>
 * <p>Erstellt von bremen online services Entwicklungs- und
 * Betriebsgesellschaft GmbH & Co. KG</p>
 * <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
 * freie Softwarebibliotheken genutzt werden.</p>Die Lizenzbestimmungen knnen
 * unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
 * Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
 * 28015 Bremen schriftlich angefordert werden.</p>
 *
 *
 * @author R. Lindemann, N. Bngener
 * @version 1.5
 */
public class SOAPFault extends OSCIResponseTo
{
  //  private static Log log = LogFactory.getLog(ResponseToExitDialog.class);
  private static final String soapFaultIntro = "<soap:Envelope xmlns:soap=\"http://schemas.xmlsoap.org/soap/envelope/\" xmlns:osci=\"http://www.osci.de/2002/04/osci\"><soap:Body><soap:Fault><faultcode>soap:";
  private static final String soapFaultExtro = "</osci:Code></detail></soap:Fault></soap:Body></soap:Envelope>";
  private String oscicode;
  private String soapfault;

  /**
   * Legt ein SOAP-Fehlerobjekt fr den genannten OSCI-Code an.
   * @param oscicode OSCI-Fehlercode (s. Spezifikation)
   */
  public SOAPFault(String oscicode)
  {
    super();
    this.oscicode = oscicode;
    messageType = RESPONSE_TO_EXIT_DIALOG;
  }

  /**
   * undocumented
   */
  protected void compose()
  {
    soapfault = soapFaultIntro;

    if (oscicode.equals("9000") || oscicode.equals("9503"))
      soapfault += "Server";
    else
      soapfault += "Client";

    soapfault += ("</faultcode><faultstring>" + DialogHandler.text.getString(oscicode) +
    "</faultstring><detail><osci:Code>" + oscicode + soapFaultExtro);
    stateOfMsg |= STATE_COMPOSED;
  }

  /**
   * undocumented
   *
   * @param out undocumented
   *
   * @throws IOException undocumented
   */
  protected void writeXML(OutputStream out) throws IOException
  {
    compose();
    out.write(("\r\nMIME-Version: 1.0\r\nContent-Type: Multipart/Related; boundary=" + DialogHandler.boundary +
              "; type=text/xml\r\n").getBytes(Constants.CHAR_ENCODING));
    out.write(("\r\n--" + DialogHandler.boundary + "\r\nContent-Type: text/xml; charset=UTF-8\r\n").getBytes(Constants.CHAR_ENCODING));
    out.write(("Content-Transfer-Encoding: 8bit\r\nContent-ID: <" + contentID + ">\r\n\r\n").getBytes(Constants.CHAR_ENCODING));
    out.write(("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\r\n\r\n").getBytes(Constants.CHAR_ENCODING));
    out.write(soapfault.getBytes(Constants.CHAR_ENCODING));
    out.write(("\r\n--" + DialogHandler.boundary + "--\r\n").getBytes(Constants.CHAR_ENCODING));
  }

  /**
   * Mit dieser Methode knnen passive Empfnger SOAP-Fehlernachrichten an
   * den Intermedir zurckschicken.
   * @param out OutputStream
   * @throws IOException bei Schreibfehlern
   */
  public void writeToStream(OutputStream out) throws IOException
  {
    writeXML(out);
  }

  /**
   * undocumented
   *
   * @param args undocumented
   */
  public static void main(String[] args)
  {
    try
    {
      SOAPFault osci = new SOAPFault("9000");
      FileOutputStream fos = new FileOutputStream("/" +
                                                  osci.getClass().getName().substring(osci.getClass().getName()
                                                                                      .lastIndexOf('.') + 1) + ".xml");
      osci.writeToStream(fos);
      fos.close();
    }
    catch (Exception ex)
    {
      ex.printStackTrace();
    }
  }
}
