package de.osci.osci12.messagetypes;

import de.osci.osci12.common.Constants;
import de.osci.osci12.messageparts.MessagePartsFactory;
import de.osci.osci12.messageparts.ProcessCardBundle;
import de.osci.osci12.messageparts.ProcessCardBundleBuilder;
import de.osci.osci12.soapheader.ControlBlockHBuilder;
import de.osci.osci12.soapheader.DesiredLanguagesH;
import de.osci.osci12.soapheader.IntermediaryCertificatesHBuilder;
import de.osci.osci12.soapheader.NonIntermediaryCertificatesHBuilder;
import de.osci.osci12.soapheader.QualityOfTimestampHBuilder;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

import java.net.URISyntaxException;


/**
 * StoredMessag-Parser.
 *
 * <p>Copyright  2010 Freie Hansestadt Bremen</p>
 * <p>Erstellt von bremen online services Entwicklungs- und
 * Betriebsgesellschaft GmbH & Co. KG</p>
 * <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
 * freie Softwarebibliotheken genutzt werden.</p>Die Lizenzbestimmungen knnen
 * unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
 * Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
 * 28015 Bremen schriftlich angefordert werden.</p>
 *
 *
 * @author R. Lindemann / N. Bngener
 * @version 1.5
 */
class StoredMessageBuilder extends OSCIMessageBuilder
{
  private static Log log = LogFactory.getLog(StoredMessageBuilder.class);
  /**
   *  Objekt ProcessCardBundle fr ProcessCard Information
   */
  private ProcessCardBundleBuilder processCardBuilder = null;
  private java.util.Vector processCardBundles = new java.util.Vector();
  private ProcessCardBundleBuilder replyProcessCardBuilder = null;

  /**
   *  Constructor for the ResponseToForwardDeliveryBuilder object
   *
   *@param  parentHandler  Description of Parameter
   *@param  msgType  Description of Parameter
   */
  public StoredMessageBuilder(OSCIEnvelopeBuilder parentHandler, int msgType)
  {
    super(parentHandler);
    msg = new de.osci.osci12.messagetypes.StoredMessage(msgType);
  }

  /**
   *  Description of the Method
   *
   *@param  uri                           Description of Parameter
   *@param  localName                     Description of Parameter
   *@param  qName                         Description of Parameter
   *@param  attributes                    Description of Parameter
   *@exception  SAXException  Description of Exception
   */
  public void startElement(String uri, String localName, String qName, Attributes attributes)
                    throws SAXException
  {
    if (log.isDebugEnabled())
      log.debug("Start-Element: " + localName + ":" + uri);

    if (localName.equals("MessageId") && uri.equals(OSCI_XMLNS))
      currentElement = new StringBuffer();
    else if (insideHeader && localName.equals("MessageIdResponse") && uri.equals(OSCI_XMLNS) &&
                 (msg.getMessageType() == OSCIMessage.PROCESS_DELIVERY))
      currentElement = new StringBuffer();
    else if (localName.equals("SelectionRule") && uri.equals(OSCI_XMLNS))
    {
    }
    else if (localName.equals("Quantity") && uri.equals(OSCI_XMLNS))
      ((StoredMessage) msg).quantityLimit = Long.parseLong(attributes.getValue("Limit"));
    else if (localName.equals("Feedback") && uri.equals(OSCI_XMLNS))
    {
      parentBuilder.xmlReader.setContentHandler(new FeedbackBuilder(this));
    }
    else if (localName.equals("ProcessCardBundle") && uri.equals(OSCI_XMLNS))
    {
      if (processCardBuilder != null)
        processCardBundles.add(processCardBuilder.getProcessCardBundleObject());

      int[] check = { -1, -1, -1, -1 };
      processCardBuilder = new ProcessCardBundleBuilder("ProcessCardBundle", parentBuilder.xmlReader, this, check);
      parentBuilder.xmlReader.setContentHandler(processCardBuilder);
    }
    else if (insideBody)
    {
      if (localName.equals("ContentPackage") && uri.equals(OSCI_XMLNS))
        parentBuilder.xmlReader.setContentHandler(MessagePartsFactory.createContentPackageBuilder(this));
    }
    else if (insideHeader)
    {
      // ### Auswerten des ControlBlock Headers###
      if (localName.equals("ControlBlock") && uri.equals(OSCI_XMLNS))
        parentBuilder.xmlReader.setContentHandler(new ControlBlockHBuilder(this, attributes,
                                                                           new int[] { -1, -1, -1, -1 }));
      // ### Auswerten des ClientSignature Headers###
      else if ((localName.equals("SupplierSignature") && uri.equals(OSCI_XMLNS)) ||
                   (localName.equals("ClientSignature") && uri.equals(OSCI_XMLNS)))
        parentBuilder.xmlReader.setContentHandler(MessagePartsFactory.createOsciSignatureBuilder(parentBuilder.xmlReader,
                                                                                                 this, attributes));
      else if (localName.equals("DesiredLanguages") && uri.equals(OSCI_XMLNS))
      {
        msg.desiredLanguagesH = new DesiredLanguagesH(attributes.getValue("LanguagesList"));
        msg.desiredLanguagesH.setRefID(attributes.getValue("Id"));
        msg.desiredLanguagesH.setNSPrefixes(msg);
        msg.dialogHandler.setLanguageList(attributes.getValue("LanguagesList"));
      }
      else if (localName.equals("QualityOfTimestamp") && uri.equals(OSCI_XMLNS))
        parentBuilder.xmlReader.setContentHandler(new QualityOfTimestampHBuilder(this, attributes));
      //
      else if (localName.equals("RequestProcessCardBundle") && uri.equals(OSCI_XMLNS))
      {
        int[] check = { -1, -1, -1, -1 };
        processCardBuilder = new ProcessCardBundleBuilder("RequestProcessCardBundle", parentBuilder.xmlReader, this,
                                                          check);
        parentBuilder.xmlReader.setContentHandler(processCardBuilder);
      }
      else if (localName.equals("ReplyProcessCardBundle") && uri.equals(OSCI_XMLNS))
      {
        int[] check = { -1, -1, -1, -1 };
        replyProcessCardBuilder = new ProcessCardBundleBuilder("ReplyProcessCardBundle", parentBuilder.xmlReader, this,
                                                               check);
        parentBuilder.xmlReader.setContentHandler(replyProcessCardBuilder);
      }
      else if (localName.equals("IntermediaryCertificates") && uri.equals(OSCI_XMLNS))
      {
        int[] check = { -1, -1 };
        parentBuilder.xmlReader.setContentHandler(new IntermediaryCertificatesHBuilder(this, attributes, check));
      }
      else if (localName.equals("NonIntermediaryCertificates") && uri.equals(OSCI_XMLNS))
      {
        int[] check = { -1, -1, -1, -1, -1, -1, -1 };
        parentBuilder.xmlReader.setContentHandler(new NonIntermediaryCertificatesHBuilder(this, attributes, check));
      }
      else if (localName.equals("Subject") && uri.equals(OSCI_XMLNS))
        currentElement = new StringBuffer();
      else if (localName.equals("ContentReceiver") && uri.equals(OSCI_XMLNS))
      {
        try
        {
          ((StoredMessage) msg).uriReceiver = new java.net.URI(attributes.getValue("URI"));
        }
        catch (URISyntaxException ex)
        {
          throw new SAXException(ex);
        }
      }
      else if (localName.equals("QualityOfTimestamp") && uri.equals(OSCI_XMLNS))
        parentBuilder.xmlReader.setContentHandler(new QualityOfTimestampHBuilder(this, attributes));
      else if ((localName.equals("mediateDelivery") || localName.equals("responseToMediateDelivery") ||
                   localName.equals("storeDelivery") || localName.equals("responseToStoreDelivery") ||
                   localName.equals("fetchDelivery") || localName.equals("responseToFetchDelivery") ||
                   localName.equals("fetchProcessCard") || localName.equals("responseToFetchProcessCard") ||
                   localName.equals("forwardDelivery") || localName.equals("responseToForwardDelivery") ||
                   localName.equals("acceptDelivery") || localName.equals("responseToAcceptDelivery") ||
                   localName.equals("processDelivery") || localName.equals("responseToProcessDelivery")) &&
                   uri.equals(OSCI_XMLNS))
      {
      }
      else
        startCustomSoapHeader(uri, localName, qName, attributes);
    }
    else
      super.startElement(uri, localName, qName, attributes);
  }

  /**
   *  Description of the Method
   *
   *@param  uri                           Description of Parameter
   *@param  localName                     Description of Parameter
   *@param  qName                         Description of Parameter
   *@exception  SAXException  Description of Exception
   */
  public void endElement(String uri, String localName, String qName)
                  throws SAXException
  {
    if (log.isDebugEnabled())
      log.debug("End-Element: " + localName);

    if (localName.equals("Envelope") && uri.equals(SOAP_XMLNS))
    {
      StoredMessage sm = ((StoredMessage) msg);

      if (processCardBuilder != null)
      {
        processCardBundles.add(processCardBuilder.getProcessCardBundleObject());
        sm.processCardBundles = ((ProcessCardBundle[]) processCardBundles.toArray(new ProcessCardBundle[0]));

        if (replyProcessCardBuilder != null)
          sm.processCardBundleReply = replyProcessCardBuilder.getProcessCardBundleObject();
      }

      for (int i = 0; i < customSoapHeader.size(); i++)
        msg.addCustomHeader((String) customSoapHeader.get(i));

      msg.dialogHandler.setCheckSignatures(false);
      parentBuilder.xmlReader.setContentHandler(parentBuilder);
    }
    else if ((localName.equals("MessageId") && uri.equals(OSCI_XMLNS)) ||
                 (localName.equals("MessageIdResponse") && uri.equals(OSCI_XMLNS)))
    {
      try
      {
        if ((msg.getMessageType() == OSCIMessage.FETCH_DELIVERY) ||
                (msg.getMessageType() == OSCIMessage.RESPONSE_TO_FETCH_DELIVERY) ||
                (msg.getMessageType() == OSCIMessage.FETCH_PROCESS_CARD) ||
                (msg.getMessageType() == OSCIMessage.RESPONSE_TO_FETCH_PROCESS_CARD))
        {
          ((StoredMessage) msg).selectionMode = OSCIMessage.SELECT_BY_MESSAGE_ID;
          ((StoredMessage) msg).selectionRule = new String(de.osci.helper.Base64.decode(currentElement.toString()),
                                                           Constants.CHAR_ENCODING);
        }
        else
        {
          msg.messageId = new String(de.osci.helper.Base64.decode(currentElement.toString()), Constants.CHAR_ENCODING);
        }
      }
      catch (Exception ex)
      {
        throw new SAXException(ex);
      }
    }
    else if (localName.equals("ReceptionOfDelivery") && uri.equals(OSCI_XMLNS))
    {
      ((StoredMessage) msg).selectionMode = OSCIMessage.SELECT_BY_DATE_OF_RECEPTION;
      ((StoredMessage) msg).selectionRule = currentElement.toString();
    }
    else if (localName.equals("Subject") && uri.equals(OSCI_XMLNS))
      ((StoredMessage) msg).subject = currentElement.toString();
    else
      super.endElement(uri, localName, qName);

    currentElement = null;
  }
}
