package de.osci.osci12.soapheader;

import de.osci.osci12.common.DialogHandler;
import de.osci.osci12.messagetypes.OSCIMessage;
import de.osci.osci12.messagetypes.OSCIMessageBuilder;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;


/**
 * <p>Copyright  2010 Freie Hansestadt Bremen</p>
 * <p>Erstellt von bremen online services Entwicklungs- und
 * Betriebsgesellschaft GmbH & Co. KG</p>
 * <p>Diese Bibliothek kann von jedermann nach Magabe der Bremer Lizenz fr
 * freie Softwarebibliotheken genutzt werden.</p>Die Lizenzbestimmungen knnen
 * unter der URL www.osci.de abgerufen oder bei der OSCI-Leitstelle,
 * Senator fr Finanzen, Freie Hansestadt Bremen, Postfach 10 15 40,
 * 28015 Bremen schriftlich angefordert werden.</p>
 *
 *
 * @author R. Lindemann, N.Bngener
 * @version 1.5
 */
public class QualityOfTimestampHBuilder extends de.osci.osci12.messageparts.MessagePartParser
{
  //  private static Log log = LogFactory.getLog(QualityOfTimestampHBuilder.class);
  QualityOfTimestampH quotsh;

  /**
   *  Constructor for the QualityOfTimestampHBuilder object
   *
   *@param  parentHandler                 Description of Parameter
   *@param  atts                          Description of Parameter
   *@exception  SAXException  Description of Exception
   */
  public QualityOfTimestampHBuilder(OSCIMessageBuilder parentHandler, Attributes atts)
  {
    super(parentHandler);
    quotsh = new QualityOfTimestampH(atts.getValue("Service").equals("reception"),
                                     atts.getValue("Quality").equals("cryptographic"));

    if (quotsh.isServiceReception())
      msg.qualityOfTimestampTypeReception = quotsh;
    else
      msg.qualityOfTimestampTypeCreation = quotsh;

    if (atts.getValue("Id") != null)
      quotsh.setRefID(atts.getValue("Id"));

    OSCIMessage msg = parentHandler.getOSCIMessage();
    quotsh.setNSPrefixes(msg);
  }

  /**
   *  Gets the qualityOfTimestampH attribute of the QualityOfTimestampHBuilder
   *  object
   *
   *@return    The qualityOfTimestampH value
   */
  public QualityOfTimestampH getQualityOfTimestampH()
  {
    return quotsh;
  }

  /**
   *  Description of the Method
   *
   *@param  uri                           Description of Parameter
   *@param  localName                     Description of Parameter
   *@param  qName                         Description of Parameter
   *@exception  SAXException  Description of Exception
   */
  public void endElement(String uri, String localName, String qName)
                  throws SAXException
  {
    if (!(localName.equals("QualityOfTimestamp") && uri.equals(OSCI_XMLNS)))
    {
      throw new SAXException(DialogHandler.text.getString("unexpected_entry") + ": " + localName);
    }

    xmlReader.setContentHandler(parentHandler);
  }
}
